from PyQt5.QtCore import pyqtSlot,  Qt
from PyQt5.QtGui import QIcon, QFont
from PyQt5.QtChart import QChart
from PyQt5.QtWidgets import  QWidget, QMenu, QProgressDialog, QVBoxLayout, QHBoxLayout, QAbstractItemView, QTableWidgetItem, QLabel
from xulpymoney.libxulpymoney import AnnualTarget, Assets, Money, AccountOperationManager, DividendHeterogeneusManager, InvestmentOperationHistoricalHeterogeneusManager, Percentage
from xulpymoney.libxulpymoneyfunctions import  list2string, none2decimal0, qcenter, qleft, qmessagebox,  day_end_from_date
from xulpymoney.libxulpymoneytypes import eQColor, eMoneyCurrency
from xulpymoney.ui.myqtablewidget import myQTableWidget
from decimal import Decimal
from xulpymoney.ui.canvaschart import VCTemporalSeries
from xulpymoney.ui.Ui_wdgTotal import Ui_wdgTotal
import datetime
import logging



class TotalMonth:
    """All values are calculated in last day of the month"""
    def __init__(self, mem, year, month):
        self.mem=mem
        self.year=year
        self.month=month
        self.expenses_value=None
        self.no_loses_value=None
        self.dividends_value=None
        self.incomes_value=None
        self.funds_revaluation_value=None
        self.gains_value=None
        self.total_accounts_value=None
        self.total_investments_value=None
        self.total_investments_high_low_value=None
        self.total_zerorisk_value=None
        self.total_bonds_value=None

    def i_d_g_e(self):
        return self.incomes()+self.dividends()+self.gains()+self.expenses()

    def d_g(self):
        """Dividends+gains"""
        return self.gains()+self.dividends()

    def expenses(self):
        if self.expenses_value==None:
            self.expenses_value=Assets(self.mem).saldo_por_tipo_operacion( self.year,self.month, 1)#La facturación de tarjeta dentro esta por el union
        return self.expenses_value

    def dividends(self):
        if self.dividends_value==None:
            self.dividends_value=Assets(self.mem).dividends_neto(  self.year, self.month)
        return self.dividends_value

    def incomes(self):
        if self.incomes_value==None:
            self.incomes_value=Assets(self.mem).saldo_por_tipo_operacion(  self.year,self.month,2)-self.dividends()
        return self.incomes_value

    def gains(self):
        if self.gains_value==None:
            self.gains_value=Assets(self.mem).consolidado_neto(self.mem.data.investments, self.year, self.month)
        return self.gains_value

    def funds_revaluation(self):
        if self.funds_revaluation_value==None:
            self.funds_revaluation_value=self.mem.data.investments_active().revaluation_monthly(2, self.year, self.month)#2 if type funds
        return self.funds_revaluation_value

    def name(self):
        return "{}-{}".format(self.year, self.month)

    def last_day(self):
        date=datetime.date(self.year, self.month, 1)
        if date.month == 12:
            return date.replace(day=31)
        return date.replace(month=date.month+1, day=1) - datetime.timedelta(days=1)

    def first_day(self):
        return datetime.date(self.year, self.month, self.day)

    def total(self):
        """Total assests in the month"""
        return self.total_accounts()+self.total_investments()

    def total_accounts(self):
        if self.total_accounts_value==None:
            self.total_accounts_value=Assets(self.mem).saldo_todas_cuentas( self.last_day())
        return self.total_accounts_value

    def total_investments(self):
        if self.total_investments_value==None:
            self.total_investments_value=Assets(self.mem).saldo_todas_inversiones(self.mem.data.investments,  self.last_day())
        return self.total_investments_value

    def total_investments_high_low(self):
        if self.total_investments_high_low_value==None:
            self.total_investments_high_low_value=Assets(self.mem).saldo_todas_inversiones_high_low(self.mem.data.investments,  self.last_day())
        return self.total_investments_high_low_value


    def total_zerorisk(self): 
        if self.total_zerorisk_value==None:
            self.total_zerorisk_value=Assets(self.mem).patrimonio_riesgo_cero(self.mem.data.investments, self.last_day())
        return self.total_zerorisk_value

    def total_bonds(self):
        if self.total_bonds_value==None:
            self.total_bonds_value=Assets(self.mem).saldo_todas_inversiones_bonds(self.last_day())
        return self.total_bonds_value

    def total_no_losses(self):
        if self.no_loses_value==None:
            self.no_loses_value=Assets(self.mem).invested(self.last_day())+self.total_accounts()
        return self.no_loses_value

class TotalYear:
    """Set of 12 totalmonths in the same year"""
    def __init__(self, mem, year):
        self.mem=mem
        self.year=year
        self.arr=[]
        self.total_last_year=Assets(self.mem).saldo_total(self.mem.data.investments,  datetime.date(self.year-1, 12, 31))
        self.generate()

    def generate(self):
        for i in range(1, 13):
            self.arr.append(TotalMonth(self.mem, self.year, i))

    def find(self, year, month):
        for m in self.arr:
            if m.year==year and m.month==month:
                return m
        return None

    def expenses(self):
        result=Money(self.mem, 0, self.mem.localcurrency)
        for m in self.arr:
            result=result+m.expenses()
        return result

    def i_d_g_e(self):
        return self.incomes()+self.dividends()+self.gains()+self.expenses()

    def funds_revaluation(self):
        return self.mem.data.investments_active().revaluation_annual(2, self.year)#2 if type funds

    def incomes(self):
        result=Money(self.mem, 0, self.mem.localcurrency)
        for m in self.arr:
            result=result+m.incomes()
        return result

    def gains(self):
        result=Money(self.mem, 0, self.mem.localcurrency)
        for m in self.arr:
            result=result+m.gains()
        return result        

    def dividends(self):
        result=Money(self.mem, 0, self.mem.localcurrency)
        for m in self.arr:
            result=result+m.dividends()
        return result

    def d_g(self):
        """Dividends+gains"""
        return self.gains()+self.dividends()

    def difference_with_previous_month(self, totalmonth):
        """Calculates difference between totalmonth and the total with previous month"""
        if totalmonth.month==1:
            totalprevious=self.total_last_year
        else:
            previous=self.find(self.year, totalmonth.month-1)
            totalprevious=previous.total()
        return totalmonth.total()-totalprevious

    def difference_with_previous_year(self):
        """Calculates difference between totalmonth of december and the total last year"""
        return self.find(self.year, 12).total()-self.total_last_year

    def assets_percentage_in_month(self, month):
        """Calculates the percentage of the assets in this month from total last year"""
        m=self.find(self.year, month)
        return Percentage(m.total()-self.total_last_year, self.total_last_year)

class TotalGraphic:
    """Set of totalmonths to generate a graphic"""
    def __init__(self, mem, startyear, startmonth):
        self.mem=mem
        self.startyear=startyear
        self.startmonth=startmonth
        self.arr=[]
        self.generate()

    def generate(self):
        date=self.previousmonth_lastday()
        while datetime.date.today()>=date:
            self.arr.append(TotalMonth(self.mem, date.year, date.month))
            date=self.nextmonth_firstday(date)#Only gets year  and month, so  I can use first day

    def find(self, year, month):
        for m in self.arr:
            if m.year==year and m.month==month:
                return m
        return None

    def length(self):
        return len(self.arr)

    def previousmonth_lastday(self, date=None):
        """If date is None, it users start year and start month"""
        if date==None:
            date=datetime.date(self.startyear, self.startmonth, 1)
        return datetime.date(date.year, date.month, 1)-datetime.timedelta(days=1)

    def nextmonth_firstday(self, date=None):
        """If date is None, date is today"""
        if date==None:
            date=datetime.date.today()

        if date.month==12:
            month=1
            year=date.year+1
        else:
            month=date.month+1
            year=date.year

        return datetime.date(year, month, 1)

class wdgTotal(QWidget, Ui_wdgTotal):
    def __init__(self, mem,  parent=None):
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem   

        fechainicio=Assets(self.mem).first_datetime_with_user_data()         

        self.setData=None#Será un TotalYear
        self.setGraphic=None #Será un TotalGraphic
        
        if fechainicio==None: #Base de datos vacía
            self.tab.setEnabled(False)
            return
        
        self.table.settings(self.mem, "wdgTotal")
        self.tblTargets.settings(self.mem, "wdgTotal")
        self.tblTargetsPlus.settings(self.mem, "wdgTotal")
        self.tblInvestOrWork.settings(self.mem,  "wdgTotal")
        self.tblMakeEndsMeet.settings(self.mem, "wdgTotal")
        
        self.annualtarget=None#AnnualTarget Object
        
        self.wyData.initiate(fechainicio.year, datetime.date.today().year, datetime.date.today().year)
        self.wyChart.initiate(fechainicio.year, datetime.date.today().year, datetime.date.today().year-3)
        self.wyChart.label.setText(self.tr("Data from selected year"))

        self.view=None#QChart view

        self.tab.setCurrentIndex(0)
        self.tabData.setCurrentIndex(0)
        self.tabPlus.setCurrentIndex(0)

        self.load_data()
        self.load_targets()
        self.load_targets_with_funds_revaluation()
        self.load_invest_or_work()
        self.load_make_ends_meet()
        self.wyData.changed.connect(self.on_wyData_mychanged)#Used my due to it took default on_wyData_changed
        self.wyChart.changed.connect(self.on_wyChart_mychanged)



    def load_data(self):
        self.table.clearContents()
        self.table.applySettings()
        inicio=datetime.datetime.now()     
        self.setData=TotalYear(self.mem, self.wyData.year)
        self.lblPreviousYear.setText(self.tr("Balance at {0}-12-31: {1}".format(self.setData.year-1, self.setData.total_last_year)))
        for i, m in enumerate(self.setData.arr):
            if m.year<datetime.date.today().year or (m.year==datetime.date.today().year and m.month<=datetime.date.today().month):
                self.table.setItem(0, i, m.incomes().qtablewidgetitem())
                self.table.setItem(1, i, m.gains().qtablewidgetitem())
                self.table.setItem(2, i, m.dividends().qtablewidgetitem())
                self.table.setItem(3, i, m.expenses().qtablewidgetitem())
                self.table.setItem(4, i, m.i_d_g_e().qtablewidgetitem())
                self.table.setItem(6, i, m.total_accounts().qtablewidgetitem())
                self.table.setItem(7, i, m.total_investments().qtablewidgetitem())
                self.table.setItem(8, i, m.total().qtablewidgetitem())
                self.table.setItem(9, i, self.setData.difference_with_previous_month(m).qtablewidgetitem())
                self.table.setItem(11, i, self.setData.assets_percentage_in_month(m.month).qtablewidgetitem())
        self.table.setItem(0, 12, self.setData.incomes().qtablewidgetitem())
        self.table.setItem(1, 12, self.setData.gains().qtablewidgetitem())
        self.table.setItem(2, 12, self.setData.dividends().qtablewidgetitem())
        self.table.setItem(3, 12, self.setData.expenses().qtablewidgetitem())
        self.table.setItem(4, 12, self.setData.i_d_g_e().qtablewidgetitem())      
        self.table.setItem(9, 12, self.setData.difference_with_previous_year().qtablewidgetitem())    
        self.table.setItem(11, 12, self.setData.assets_percentage_in_month(12).qtablewidgetitem())
        self.table.setCurrentCell(6, datetime.date.today().month-1)
        s=""
        s=self.tr("This year I've generated {}.").format(self.setData.gains()+self.setData.dividends())
        invested=Assets(self.mem).invested(datetime.date.today())
        current=Assets(self.mem).saldo_todas_inversiones(self.mem.data.investments, datetime.date.today())
        s=s+"\n"+self.tr("Difference between invested amount and current invesment balance: {} - {} = {}").format(invested,  current,  current-invested)
        self.lblInvested.setText(s)
        final=datetime.datetime.now()          
        logging.info("wdgTotal > load_data: {0}".format(final-inicio))

    def load_targets(self):
        self.annualtarget=AnnualTarget(self.mem).init__from_db(self.wyData.year) 
        self.lblTarget.setText(self.tr("Annual target percentage of total assests balance at {}-12-31 ( {} )".format(self.annualtarget.year-1, self.annualtarget.lastyear_assests)))
        self.spinTarget.setValue(float(self.annualtarget.percentage))
        self.tblTargets.clearContents()
        self.tblTargets.applySettings()
        inicio=datetime.datetime.now()     
        sumd_g=Money(self.mem, 0, self.mem.localcurrency)
        for i in range(1, 13): 
            m=self.setData.find(self.setData.year, i)
            sumd_g=sumd_g+m.d_g()
            self.tblTargets.setItem(0, i-1, self.mem.localcurrency.qtablewidgetitem(self.annualtarget.monthly_balance()))
            self.tblTargets.setItem(1, i-1, self.mem.localcurrency.qtablewidgetitem_with_target(m.d_g().amount, self.annualtarget.monthly_balance()))
            self.tblTargets.setItem(3, i-1, self.mem.localcurrency.qtablewidgetitem(self.annualtarget.monthly_balance()*i))
            self.tblTargets.setItem(4, i-1, self.mem.localcurrency.qtablewidgetitem_with_target(sumd_g.amount, self.annualtarget.monthly_balance()*i))
        self.tblTargets.setItem(0, 12, self.mem.localcurrency.qtablewidgetitem(self.annualtarget.annual_balance()))
        self.tblTargets.setItem(1, 12, self.mem.localcurrency.qtablewidgetitem_with_target(sumd_g.amount, self.annualtarget.annual_balance()))
        self.tblTargets.setCurrentCell(2, datetime.date.today().month-1)   
                
        s=""
        s=s+self.tr("This report shows if the user reaches the annual and monthly target.") +"\n\n"
        s=s+self.tr("Total gains are the result of adding dividends to gains")+"\n\n"
        s=s+self.tr("The cumulative target row shows compliance of the target in the year.")+"\n\n"
        s=s+self.tr("Green color shows that target has been reached.")
        self.lblTargets.setText(s)
        
        logging.info("wdgTargets > load_data_targets: {0}".format(datetime.datetime.now()  -inicio))
        
    def load_targets_with_funds_revaluation(self):
        self.tblTargetsPlus.clearContents()
        self.tblTargetsPlus.applySettings()
        inicio=datetime.datetime.now()     

        sumd_g=Money(self.mem, 0, self.mem.localcurrency)
        sumf=Money(self.mem, 0, self.mem.localcurrency)
        for i in range(1, 13): 
            m=self.setData.find(self.setData.year, i)
            sumd_g=sumd_g+m.d_g()
            sumf=sumf+m.funds_revaluation()
            self.tblTargetsPlus.setItem(0, i-1, self.mem.localcurrency.qtablewidgetitem(self.annualtarget.monthly_balance()))
            self.tblTargetsPlus.setItem(1, i-1,m.d_g().qtablewidgetitem())
            self.tblTargetsPlus.setItem(2, i-1, m.funds_revaluation().qtablewidgetitem())
            self.tblTargetsPlus.setItem(3, i-1, self.mem.localcurrency.qtablewidgetitem_with_target(m.d_g().amount+m.funds_revaluation().amount, self.annualtarget.monthly_balance()))
            
            self.tblTargetsPlus.setItem(5, i-1, self.mem.localcurrency.qtablewidgetitem(self.annualtarget.monthly_balance()*i))
            self.tblTargetsPlus.setItem(6, i-1, self.mem.localcurrency.qtablewidgetitem_with_target(sumd_g.amount+sumf.amount, self.annualtarget.monthly_balance()*i))
        self.tblTargetsPlus.setItem(0, 12, self.mem.localcurrency.qtablewidgetitem(self.annualtarget.annual_balance()))
        self.tblTargetsPlus.setItem(1, 12, sumd_g.qtablewidgetitem())
        self.tblTargetsPlus.setItem(2, 12, sumf.qtablewidgetitem())
        self.tblTargetsPlus.setItem(3, 12, self.mem.localcurrency.qtablewidgetitem_with_target(sumd_g.amount+sumf.amount,self.annualtarget.annual_balance()))
        self.tblTargetsPlus.setCurrentCell(2, datetime.date.today().month-1)   
                
        s=""
        s=s+self.tr("This report shows if the user reaches the annual and monthly target.") +"\n\n"
        s=s+self.tr("Total is the result of adding dividends to gain and funds revaluation")+"\n\n"
        s=s+self.tr("The cumulative target row shows compliance of the target in the year.")+"\n\n"
        s=s+self.tr("Green color shows that target has been reached.")
        self.lblTargetsPlus.setText(s)
        
        logging.info("wdgTargets > load_data_targets_with_funds_revaluation: {0}".format(datetime.datetime.now()  -inicio))

    def load_invest_or_work(self):
        def qresult(dg_e):
            """Returns a qtablewidgetitem with work or invest
            dg_e=dividends+gains-expenses
            dg_i=dividends+gains-incomes
            """
            item=qcenter("")
            if dg_e.isZero():
                return item
            if not dg_e.isGETZero():
                item.setText(self.tr("Work"))
                item.setBackground(eQColor.Red)
            else:
                item.setText(self.tr("Invest"))
                item.setBackground(eQColor.Green)
            return item            
        ##------------------------------------------------
        inicio=datetime.datetime.now()    
        self.tblInvestOrWork.clearContents()
        self.tblInvestOrWork.applySettings()
        for i in range(1, 13): 
            m=self.setData.find(self.setData.year, i)
            self.tblInvestOrWork.setItem(0, i-1, m.d_g().qtablewidgetitem())
            self.tblInvestOrWork.setItem(1, i-1, m.expenses().qtablewidgetitem())
            self.tblInvestOrWork.setItem(3, i-1, (m.d_g()+m.expenses()).qtablewidgetitem())#Es mas porque es - y gastos -
            self.tblInvestOrWork.setItem(5, i-1, qresult(m.d_g()+m.expenses()))
        self.tblInvestOrWork.setItem(0, 12, self.setData.d_g().qtablewidgetitem())
        self.tblInvestOrWork.setItem(1, 12, self.setData.expenses().qtablewidgetitem())
        self.tblInvestOrWork.setItem(3, 12, (self.setData.d_g()+self.setData.expenses()).qtablewidgetitem())
        self.tblInvestOrWork.setItem(5, 12, qresult(self.setData.d_g()+self.setData.expenses()))
        self.tblInvestOrWork.setCurrentCell(2, datetime.date.today().month-1)   
        
        s=""
        s=s+self.tr("This report shows if the user could retire due to its investments") +"\n\n"
        s=s+self.tr("Total gains are the result of adding dividends to gains")+"\n\n"
        s=s+self.tr("Difference between total gains and expenses shows if user could cover his expenses with his total gains")+"\n\n"
        s=s+self.tr("Investment taxes are not evaluated in this report")
        self.lblInvestOrWork.setText(s)
        logging.info ("wdgTotal > load invest or work: {0}".format(datetime.datetime.now()  -inicio))

    def load_make_ends_meet(self):
        def qresult(res):
            """Returns a qtablewidgetitem with yes or no
            """
            item=qcenter("")
            if res.isZero():
                return item
            if not res.isGETZero():
                item.setText(self.tr("No"))
                item.setBackground(eQColor.Red)
            else:
                item.setText(self.tr("Yes"))
                item.setBackground(eQColor.Green)
            return item            
        ##------------------------------------------------
        inicio=datetime.datetime.now()    
        self.tblMakeEndsMeet.clearContents()
        self.tblMakeEndsMeet.applySettings()
        for i in range(1, 13): 
            m=self.setData.find(self.setData.year, i)
            self.tblMakeEndsMeet.setItem(0, i-1, m.incomes().qtablewidgetitem())
            self.tblMakeEndsMeet.setItem(1, i-1, m.expenses().qtablewidgetitem())
            self.tblMakeEndsMeet.setItem(3, i-1, (m.incomes()+m.expenses()).qtablewidgetitem())#Es mas porque es - y gastos -
            self.tblMakeEndsMeet.setItem(5, i-1, qresult(m.incomes()+m.expenses()))
        self.tblMakeEndsMeet.setItem(0, 12, self.setData.incomes().qtablewidgetitem())
        self.tblMakeEndsMeet.setItem(1, 12, self.setData.expenses().qtablewidgetitem())
        self.tblMakeEndsMeet.setItem(3, 12, (self.setData.incomes()+self.setData.expenses()).qtablewidgetitem())
        self.tblMakeEndsMeet.setItem(5, 12, qresult(self.setData.incomes()+self.setData.expenses()))
        self.tblMakeEndsMeet.setCurrentCell(2, datetime.date.today().month-1)   
        
        s=""
        s=s+self.tr("This report shows if the user makes ends meet") +"\n\n"
        s=s+self.tr("Difference between incomes and expenses shows if user could cover his expenses with his incomes")
        self.lblMakeEndsMeet.setText(s)
        logging.info("wdgTotal > load_make_ends_meet: {0}".format(datetime.datetime.now()  -inicio))


    def load_graphic(self, animations=True):               
        inicio=datetime.datetime.now()  
        
        self.setGraphic=TotalGraphic(self.mem, self.wyChart.year, 1)

        if self.view!=None:
            self.tabGraphTotal.removeWidget(self.view)
            self.view.close()
        self.view=VCTemporalSeries()
        
        if animations==False:
            self.view.chart.setAnimationOptions(QChart.NoAnimation)
                #Series creation
        last=self.setGraphic.find(datetime.date.today().year, datetime.date.today().month)
        lsNoLoses=self.view.appendTemporalSeries(self.tr("Total without losses assets")+": {}".format(last.total_no_losses()))
        lsMain=self.view.appendTemporalSeries(self.tr("Total assets")+": {}".format(last.total()))
        lsZero=self.view.appendTemporalSeries(self.tr("Zero risk assets")+": {}".format(last.total_zerorisk()))
        lsBonds=self.view.appendTemporalSeries(self.tr("Bond assets")+": {}".format(last.total_bonds()))
        lsRisk=self.view.appendTemporalSeries(self.tr("Risk assets")+": {}".format(last.total()-last.total_zerorisk()-last.total_bonds()))
                
        
        progress = QProgressDialog(self.tr("Filling report data"), self.tr("Cancel"), 0,self.setGraphic.length())
        progress.setModal(True)
        progress.setWindowTitle(self.tr("Calculating data..."))
        progress.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
        for m in self.setGraphic.arr:
            if progress.wasCanceled():
                break
            progress.setValue(progress.value()+1)
            epoch=day_end_from_date(m.last_day(), self.mem.localzone)
            total=m.total().amount
            zero=m.total_zerorisk().amount
            bonds=m.total_bonds().amount
            self.view.appendTemporalSeriesData(lsMain, epoch, m.total().amount)
            self.view.appendTemporalSeriesData(lsZero, epoch, m.total_zerorisk().amount)
            self.view.appendTemporalSeriesData(lsBonds, epoch, m.total_bonds().amount)
            self.view.appendTemporalSeriesData(lsRisk, epoch, total-zero-bonds)
            self.view.appendTemporalSeriesData(lsNoLoses, epoch, m.total_no_losses().amount)
        self.view.display()
        
        self.tabGraphTotal.addWidget(self.view)

            
        
        logging.info("wdgTotal > load_graphic: {0}".format(datetime.datetime.now()-inicio))



    def on_wyData_mychanged(self):
        self.load_data()    
        self.load_targets()
        self.load_targets_with_funds_revaluation()
        self.load_invest_or_work()
        self.load_make_ends_meet()

    def on_wyChart_mychanged(self):
        self.load_graphic()      
        
    def on_cmdTargets_released(self):
        self.annualtarget.percentage=self.spinTarget.value()
        self.annualtarget.save()
        self.mem.con.commit()
        self.load_targets()
        self.load_targets_with_funds_revaluation()

    def on_tab_currentChanged(self, index):
        if  index==1 and self.view==None: #If has not been plotted, plots it.
            self.on_wyChart_mychanged()
        
    @pyqtSlot() 
    def on_actionShowIncomes_triggered(self):
        
        newtab = QWidget()
        horizontalLayout = QHBoxLayout(newtab)
        table = myQTableWidget(newtab)
        table.settings(self.mem, "wdgTotal","tblShowIncomes")
        table.setEditTriggers(QAbstractItemView.NoEditTriggers)
        table.setSelectionMode(QAbstractItemView.SingleSelection)
        table.verticalHeader().setVisible(False)
        
        id_tiposoperaciones=2
        set=AccountOperationManager(self.mem)
        if self.month==13:#Year
            tabtitle=self.tr("Incomes of {}").format(self.wyData.year)
            set.load_from_db("""select id_opercuentas, datetime, id_conceptos, id_tiposoperaciones, importe, comentario, id_cuentas 
                                                    from opercuentas 
                                                    where id_tiposoperaciones={0} and 
                                                        date_part('year',datetime)={1} and
                                                        id_conceptos not in ({2}) 
                                                union all select id_opercuentas, datetime, id_conceptos, id_tiposoperaciones, importe, comentario, id_cuentas 
                                                    from opertarjetas,tarjetas 
                                                    where opertarjetas.id_tarjetas=tarjetas.id_tarjetas and 
                                                        id_tiposoperaciones={0} and 
                                                        date_part('year',datetime)={1}""".format (id_tiposoperaciones, self.wyData.year, list2string(self.mem.conceptos.considered_dividends_in_totals())))
        else:#Month
            tabtitle=self.tr("Incomes of {0} of {1}").format(self.table.horizontalHeaderItem(self.month-1).text(), self.wyData.year)
            set.load_from_db("""select id_opercuentas, datetime, id_conceptos, id_tiposoperaciones, importe, comentario, id_cuentas 
                                                    from opercuentas 
                                                    where id_tiposoperaciones={0} and 
                                                        date_part('year',datetime)={1} and 
                                                        date_part('month',datetime)={2} and 
                                                        id_conceptos not in ({3}) 
                                                union all select id_opercuentas, datetime, id_conceptos, id_tiposoperaciones, importe, comentario, id_cuentas 
                                                    from opertarjetas,tarjetas 
                                                    where opertarjetas.id_tarjetas=tarjetas.id_tarjetas and 
                                                        id_tiposoperaciones={0} and 
                                                        date_part('year',datetime)={1} and 
                                                        date_part('month',datetime)={2}""".format (id_tiposoperaciones, self.wyData.year, self.month, list2string(self.mem.conceptos.considered_dividends_in_totals())))
        set.myqtablewidget(table,  True)
        horizontalLayout.addWidget(table)
        self.tab.addTab(newtab, tabtitle)
        self.tab.setCurrentWidget(newtab)            

       
    @pyqtSlot() 
    def on_actionShowExpenses_triggered(self):     
        newtab = QWidget()
        horizontalLayout = QHBoxLayout(newtab)
        table = myQTableWidget(newtab)
        table.settings(self.mem, "wdgTotal","tblShowExpenses")
        table.setEditTriggers(QAbstractItemView.NoEditTriggers)
        table.setSelectionMode(QAbstractItemView.SingleSelection)
        table.verticalHeader().setVisible(False)
        
        id_tiposoperaciones=1
        set=AccountOperationManager(self.mem)
        if self.month==13:#Year
            tabtitle=self.tr("Expenses of {0}").format(self.wyData.year)
            set.load_from_db("""select id_opercuentas, datetime, id_conceptos, id_tiposoperaciones, importe, comentario, id_cuentas , -1 as id_tarjetas 
                                                from opercuentas 
                                                where id_tiposoperaciones={0} and 
                                                           date_part('year',datetime)={1} 
                                                union all 
                                                select id_opercuentas, datetime, id_conceptos, id_tiposoperaciones, importe, comentario, id_cuentas ,tarjetas.id_tarjetas as id_tarjetas 
                                                from opertarjetas,tarjetas 
                                                where opertarjetas.id_tarjetas=tarjetas.id_tarjetas and 
                                                            id_tiposoperaciones={0} and 
                                                            date_part('year',datetime)={1}""".format (id_tiposoperaciones, self.wyData.year))
        else:#Month
            tabtitle=self.tr("Expenses of {0} of {1}").format(self.table.horizontalHeaderItem(self.month-1).text(), self.wyData.year)
            set.load_from_db("""select id_opercuentas, datetime, id_conceptos, id_tiposoperaciones, importe, comentario, id_cuentas , -1 as id_tarjetas 
                                                from opercuentas 
                                                where id_tiposoperaciones={0} and 
                                                           date_part('year',datetime)={1} and 
                                                           date_part('month',datetime)={2} 
                                                union all 
                                                select id_opercuentas, datetime, id_conceptos, id_tiposoperaciones, importe, comentario, id_cuentas ,tarjetas.id_tarjetas as id_tarjetas 
                                                from opertarjetas,tarjetas 
                                                where opertarjetas.id_tarjetas=tarjetas.id_tarjetas and 
                                                            id_tiposoperaciones={0} and 
                                                            date_part('year',datetime)={1} and 
                                                            date_part('month',datetime)={2}""".format (id_tiposoperaciones, self.wyData.year, self.month))
        set.myqtablewidget(table,  True)
        horizontalLayout.addWidget(table)
        self.tab.addTab(newtab, tabtitle)
        self.tab.setCurrentWidget(newtab)

    @pyqtSlot() 
    def on_actionShowSellingOperations_triggered(self):
        def show_all():
            newtab = QWidget()
            horizontalLayout = QVBoxLayout(newtab)
            table = myQTableWidget(newtab)
            table.settings(self.mem, "wdgTotal","tblShowShellingOperations")
            table.setEditTriggers(QAbstractItemView.NoEditTriggers)
            table.setSelectionMode(QAbstractItemView.SingleSelection)
            table.verticalHeader().setVisible(False)
            
            positive=Money(self.mem, 0, self.mem.localcurrency)
            negative=Money(self.mem, 0, self.mem.localcurrency)
            lbl=QLabel(newtab)
            
            set=InvestmentOperationHistoricalHeterogeneusManager(self.mem)
            for i in self.mem.data.investments.arr:
                for o in i.op_historica.arr:
                    if self.month==13:#Year
                        tabtitle=self.tr("Selling operations of {0}").format(self.wyData.year)
                        if o.fecha_venta.year==self.wyData.year:
                            set.arr.append(o)
                            if o.consolidado_bruto().isGETZero():
                                positive=positive+o.consolidado_bruto().local()
                            else:
                                negative=negative+o.consolidado_bruto().local()
                    else:#Month
                        tabtitle=self.tr("Selling operations of {0} of {1}").format(self.table.horizontalHeaderItem(self.month-1).text(), self.wyData.year)
                        if o.fecha_venta.year==self.wyData.year and o.fecha_venta.month==self.month:#Venta y traspaso fondos inversion
                            set.arr.append(o)
                            if o.consolidado_bruto().isGETZero():
                                positive=positive+o.consolidado_bruto().local()
                            else:
                                negative=negative+o.consolidado_bruto().local()
            set.order_by_fechaventa()
            set.myqtablewidget(table)
            horizontalLayout.addWidget(table)
            lbl.setText(self.tr("Positive gross selling operations: {}. Negative gross selling operations: {}.").format(positive, negative))
            horizontalLayout.addWidget(lbl)
            self.tab.addTab(newtab, tabtitle)
            self.tab.setCurrentWidget(newtab)

        def show_more():
            newtab = QWidget()
            horizontalLayout = QVBoxLayout(newtab)
            table = myQTableWidget(newtab)
            table.setObjectName("tblShowShellingOperations")
            table.setEditTriggers(QAbstractItemView.NoEditTriggers)
            table.setSelectionMode(QAbstractItemView.SingleSelection)
            table.verticalHeader().setVisible(False)
            
            positive=Decimal(0)
            negative=Decimal(0)
            lbl=QLabel(newtab)
            
            set=InvestmentOperationHistoricalHeterogeneusManager(self.mem)
            for i in self.mem.data.investments.arr:
                for o in i.op_historica.arr:
                    if self.month==13:#Year
                        tabtitle=self.tr("Selling operations of {0}  (Sold after a year)").format(self.wyData.year)
                        if o.fecha_venta.year==self.wyData.year and o.tipooperacion.id in (5, 8)  and o.less_than_a_year()==False:#Venta y traspaso fondos inversion
                            set.arr.append(o)
                            if o.consolidado_bruto()>=0:
                                positive=positive+o.consolidado_bruto()
                            else:
                                negative=negative+o.consolidado_bruto()
                    else:#Month
                        tabtitle=self.tr("Selling operations of {0} of {1} (Sold after a year)").format(self.table.horizontalHeaderItem(self.month-1).text(), self.wyData.year)
                        if o.fecha_venta.year==self.wyData.year and o.fecha_venta.month==self.month and o.tipooperacion.id in (5, 8)  and o.less_than_a_year()==False:#Venta y traspaso fondos inversion
                            set.arr.append(o)
                            if o.consolidado_bruto()>=0:
                                positive=positive+o.consolidado_bruto()
                            else:
                                negative=negative+o.consolidado_bruto()
            set.order_by_fechaventa()
            set.myqtablewidget(table, "wdgTotal")
            horizontalLayout.addWidget(table)
            lbl.setText(self.tr("Positive gross selling operations: {}. Negative gross selling operations: {}.").format(self.mem.localcurrency.string(positive), self.mem.localcurrency.string(negative)))
            horizontalLayout.addWidget(lbl)
            self.tab.addTab(newtab, tabtitle)
            self.tab.setCurrentWidget(newtab)
        def show_less():
            newtab = QWidget()
            horizontalLayout = QVBoxLayout(newtab)
            table = myQTableWidget(newtab)
            table.setObjectName("tblShowShellingOperations")
            table.setEditTriggers(QAbstractItemView.NoEditTriggers)
            table.setSelectionMode(QAbstractItemView.SingleSelection)
            table.verticalHeader().setVisible(False)
            
            positive=Decimal(0)
            negative=Decimal(0)
            lbl=QLabel(newtab)
            
            set=InvestmentOperationHistoricalHeterogeneusManager(self.mem)
            for i in self.mem.data.investments.arr:
                for o in i.op_historica.arr:
                    if self.month==13:#Year
                        tabtitle=self.tr("Selling operations of {0} (Sold before a year)").format(self.wyData.year)
                        if o.fecha_venta.year==self.wyData.year and o.tipooperacion.id in (5, 8) and o.less_than_a_year()==True:#Venta y traspaso fondos inversion
                            set.arr.append(o)
                            if o.consolidado_bruto()>=0:
                                positive=positive+o.consolidado_bruto()
                            else:
                                negative=negative+o.consolidado_bruto()
                    else:#Month
                        tabtitle=self.tr("Selling operations of {0} of {1} (Sold before a year)").format(self.table.horizontalHeaderItem(self.month-1).text(), self.wyData.year)
                        if o.fecha_venta.year==self.wyData.year and o.fecha_venta.month==self.month and o.tipooperacion.id in (5, 8) and o.less_than_a_year()==True:#Venta y traspaso fondos inversion
                            set.arr.append(o)
                            if o.consolidado_bruto()>=0:
                                positive=positive+o.consolidado_bruto()
                            else:
                                negative=negative+o.consolidado_bruto()
            set.order_by_fechaventa()
            set.myqtablewidget(table, "wdgTotal")
            horizontalLayout.addWidget(table)
            lbl.setText(self.tr("Positive gross selling operations: {}. Negative gross selling operations: {}.").format(self.mem.localcurrency.string(positive), self.mem.localcurrency.string(negative)))
            horizontalLayout.addWidget(lbl)
            self.tab.addTab(newtab, tabtitle)
            self.tab.setCurrentWidget(newtab)            
        ##################################
        if self.mem.gainsyear==True:
            show_less()
            show_more()
        else:
            show_all()

    @pyqtSlot() 
    def on_actionShowDividends_triggered(self):
        newtab = QWidget()
        horizontalLayout = QHBoxLayout(newtab)
        table = myQTableWidget(newtab)
        table.settings(self.mem,"wdgTotal","tblShowDividends")
        table.setEditTriggers(QAbstractItemView.NoEditTriggers)
        table.setSelectionMode(QAbstractItemView.SingleSelection)
        table.verticalHeader().setVisible(False)
        
        set=DividendHeterogeneusManager(self.mem)
        if self.month==13:#Year
            tabtitle=self.tr("Dividends of {0}").format(self.wyData.year)
            set.load_from_db("select * from dividends where id_conceptos not in (63) and date_part('year',fecha)={0}".format (self.wyData.year))
        else:#Month
            tabtitle=self.tr("Dividends of {0} of {1}").format(self.table.horizontalHeaderItem(self.month-1).text(), self.wyData.year)
            set.load_from_db("select * from dividends where id_conceptos not in (63) and date_part('year',fecha)={0} and date_part('month',fecha)={1}".format (self.wyData.year, self.month))
        set.order_by_datetime()
        set.myqtablewidget(table,  True)
        horizontalLayout.addWidget(table)
        self.tab.addTab(newtab, tabtitle)
        self.tab.setCurrentWidget(newtab)            


    @pyqtSlot() 
    def on_actionShowComissions_triggered(self):
        newtab = QWidget()
        vlayout = QVBoxLayout(newtab)
        table = myQTableWidget(newtab)
        table.settings(self.mem,"wdgTotal","tblShowComissions")
        table.setEditTriggers(QAbstractItemView.NoEditTriggers)
        table.setSelectionMode(QAbstractItemView.SingleSelection)
        
        table.setColumnCount(13)
        table.setHorizontalHeaderItem(0, QTableWidgetItem(self.tr( "January" )))
        table.setHorizontalHeaderItem(1, QTableWidgetItem(self.tr( "February" )))
        table.setHorizontalHeaderItem(2, QTableWidgetItem(self.tr( "March" )))
        table.setHorizontalHeaderItem(3, QTableWidgetItem(self.tr( "April" )))
        table.setHorizontalHeaderItem(4, QTableWidgetItem(self.tr( "May" )))
        table.setHorizontalHeaderItem(5, QTableWidgetItem(self.tr( "June" )))
        table.setHorizontalHeaderItem(6, QTableWidgetItem(self.tr( "July" )))
        table.setHorizontalHeaderItem(7, QTableWidgetItem(self.tr( "August" )))
        table.setHorizontalHeaderItem(8, QTableWidgetItem(self.tr( "September" )))
        table.setHorizontalHeaderItem(9, QTableWidgetItem(self.tr( "October" )))
        table.setHorizontalHeaderItem(10, QTableWidgetItem(self.tr( "November" )))
        table.setHorizontalHeaderItem(11, QTableWidgetItem(self.tr( "December" )))
        table.setHorizontalHeaderItem(12, QTableWidgetItem(self.tr( "Total" )))
        
        table.setRowCount(4)
        table.setVerticalHeaderItem(0, QTableWidgetItem(self.tr( "Bank comissions" )))
        table.setVerticalHeaderItem(1, QTableWidgetItem(self.tr( "Custody fees" )))
        table.setVerticalHeaderItem(2, QTableWidgetItem(self.tr( "Invesment operation comissions" )))
        table.setVerticalHeaderItem(3, QTableWidgetItem(self.tr( "Total" )))
        table.applySettings()
        (sum_bank_comissions, sum_custody_fees, sum_investment_comissions)=(Decimal("0"), Decimal("0"), Decimal("0"))

        for column in range (12):
            bank_comissions=none2decimal0(self.mem.con.cursor_one_row("""select sum(importe) 
                                                                                                            from opercuentas 
                                                                                                            where id_conceptos=%s and 
                                                                                                                date_part('year',datetime)=%s and 
                                                                                                                date_part('month',datetime)=%s;""", (38, self.wyData.year, column+1))[0])
            table.setItem(0, column, self.mem.localcurrency.qtablewidgetitem(bank_comissions))    
            sum_bank_comissions=sum_bank_comissions+bank_comissions
            
            custody_fees=none2decimal0(self.mem.con.cursor_one_row("""select sum(importe) 
                                                                                                            from opercuentas 
                                                                                                            where id_conceptos=%s and 
                                                                                                                date_part('year',datetime)=%s and 
                                                                                                                date_part('month',datetime)=%s;""", (59, self.wyData.year, column+1))[0])
            table.setItem(1, column, self.mem.localcurrency.qtablewidgetitem(custody_fees))    
            sum_custody_fees=sum_custody_fees+custody_fees
            
            investment_comissions=-none2decimal0(self.mem.con.cursor_one_row("""select sum(comision) 
                                                                                                            from operinversiones  
                                                                                                            where date_part('year',datetime)=%s and 
                                                                                                                date_part('month',datetime)=%s;""", (self.wyData.year, column+1))[0])
            table.setItem(2, column, self.mem.localcurrency.qtablewidgetitem(investment_comissions))    
            sum_investment_comissions=sum_investment_comissions+investment_comissions
            
            table.setItem(3, column, self.mem.localcurrency.qtablewidgetitem(bank_comissions+custody_fees+investment_comissions))    
        table.setItem(0, 12, self.mem.localcurrency.qtablewidgetitem(sum_bank_comissions))    
        table.setItem(1, 12, self.mem.localcurrency.qtablewidgetitem(sum_custody_fees))    
        table.setItem(2, 12, self.mem.localcurrency.qtablewidgetitem(sum_investment_comissions))    
        table.setItem(3, 12, self.mem.localcurrency.qtablewidgetitem(sum_bank_comissions+sum_custody_fees+sum_investment_comissions))    
        vlayout.addWidget(table)

        #Number of operations
        num_operations=0
        settypes=self.mem.types.with_operation_comissions_types()
        for i, type in enumerate(settypes.arr):
            for inv in self.mem.data.investments.arr:
                if inv.product.type.id==type.id:
                    for o in inv.op.arr:
                        if o.datetime.year==self.wyData.year and o.tipooperacion.id in (4, 5):#Purchase and sale
                            num_operations=num_operations+1            
        if num_operations>0:
            label=QLabel(newtab)
            font = QFont()
            font.setPointSize(8)
            font.setBold(True)
            font.setWeight(75)
            label.setFont(font)
            label.setAlignment(Qt.AlignCenter)
            cs=self.mem.localcurrency.string
            label.setText(self.tr("Number of purchase and sale investment operations: {}. Comissions average: {}".format(int(num_operations), cs(sum_investment_comissions/num_operations))))
            vlayout.addWidget(label)
            
        self.tab.addTab(newtab, self.tr("Commision report of {}").format(self.wyData.year))
        self.tab.setCurrentWidget(newtab)        
        

    @pyqtSlot() 
    def on_actionGainsByProductType_triggered(self):
        newtab = QWidget()
        vlayout = QVBoxLayout(newtab)
        table = myQTableWidget(newtab)
        table.settings(self.mem,"wdgTotal","tblGainsByProductType")
        table.setEditTriggers(QAbstractItemView.NoEditTriggers)
        table.setSelectionMode(QAbstractItemView.SingleSelection)
        
        table.setColumnCount(3)
        
        table.setHorizontalHeaderItem(0, QTableWidgetItem(self.tr( "Product type" )))
        table.setHorizontalHeaderItem(1, QTableWidgetItem(self.tr( "Brut gains" )))
        table.setHorizontalHeaderItem(2, QTableWidgetItem(self.tr("Brut dividends")))
        table.applySettings()
        sum_gains=Money(self.mem)
        sum_dividens=Money(self.mem)
        
        settypes=self.mem.types.investment_types()
        table.setRowCount(settypes.length()+1)
        
        for i, type in enumerate(settypes.arr):
            table.setItem(i, 0, qleft(type.name))    
            gains=Money(self.mem,  0,  self.mem.localcurrency)
            dividends=Money(self.mem,  0,  self.mem.localcurrency)
            for inv in self.mem.data.investments.arr:
                if inv.product.type.id==type.id:
                    #gains
                    for o in inv.op_historica.arr:
                        if o.fecha_venta.year==self.wyData.year:
                            gains=gains+o.consolidado_bruto(eMoneyCurrency.User)
                    #dividends
                    setdiv=DividendHeterogeneusManager(self.mem)
                    setdiv.load_from_db(self.mem.con.mogrify("select * from dividends where id_inversiones=%s and fecha>=%s and fecha<=%s order by fecha", (inv.id, datetime.datetime(self.wyData.year, 1, 1, 0, 0, 0), datetime.datetime(self.wyData.year+1, 12, 31, 23, 59, 59))))
                    dividends=dividends+setdiv.gross().local()
            table.setItem(i, 1, gains.qtablewidgetitem())
            table.setItem(i, 2, dividends.qtablewidgetitem())
            sum_gains=sum_gains+gains
            sum_dividens=sum_dividens+dividends
            
        table.setItem(i+1, 0, qleft(self.tr("Total")))
        table.setItem(i+1, 1, sum_gains.qtablewidgetitem())
        table.setItem(i+1, 2, sum_dividens.qtablewidgetitem())
        
        label=QLabel(newtab)
        font = QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        label.setFont(font)
        label.setAlignment(Qt.AlignCenter)
        label.setText(self.tr("Gains + Dividends: {} + {} = {}".format(sum_gains, sum_dividens, sum_gains+sum_dividens)))
            
        vlayout.addWidget(table)
        vlayout.addWidget(label)
        self.tab.addTab(newtab, self.tr("Gains by product type of {}").format(self.wyData.year))
        self.tab.setCurrentWidget(newtab)        
            
    @pyqtSlot() 
    def on_actionShowTaxes_triggered(self):
        newtab = QWidget()
        horizontalLayout = QVBoxLayout(newtab)
        table = myQTableWidget(newtab)
        table.settings(self.mem,"wdgTotal","tblShowPaidTaxes")
        table.setEditTriggers(QAbstractItemView.NoEditTriggers)
        table.setSelectionMode(QAbstractItemView.SingleSelection)
        
        table.setColumnCount(13)
        table.setHorizontalHeaderItem(0, QTableWidgetItem(self.tr( "January" )))
        table.setHorizontalHeaderItem(1, QTableWidgetItem(self.tr( "February" )))
        table.setHorizontalHeaderItem(2, QTableWidgetItem(self.tr( "March" )))
        table.setHorizontalHeaderItem(3, QTableWidgetItem(self.tr( "April" )))
        table.setHorizontalHeaderItem(4, QTableWidgetItem(self.tr( "May" )))
        table.setHorizontalHeaderItem(5, QTableWidgetItem(self.tr( "June" )))
        table.setHorizontalHeaderItem(6, QTableWidgetItem(self.tr( "July" )))
        table.setHorizontalHeaderItem(7, QTableWidgetItem(self.tr( "August" )))
        table.setHorizontalHeaderItem(8, QTableWidgetItem(self.tr( "September" )))
        table.setHorizontalHeaderItem(9, QTableWidgetItem(self.tr( "October" )))
        table.setHorizontalHeaderItem(10, QTableWidgetItem(self.tr( "November" )))
        table.setHorizontalHeaderItem(11, QTableWidgetItem(self.tr( "December" )))
        table.setHorizontalHeaderItem(12, QTableWidgetItem(self.tr( "Total" )))
        
        table.setRowCount(5)
        table.setVerticalHeaderItem(0, QTableWidgetItem(self.tr( "Investment operation retentions" )))
        table.setVerticalHeaderItem(1, QTableWidgetItem(self.tr( "Dividend retentions" )))
        table.setVerticalHeaderItem(2, QTableWidgetItem(self.tr( "Other paid taxes" )))
        table.setVerticalHeaderItem(3, QTableWidgetItem(self.tr( "Returned taxes" )))
        table.setVerticalHeaderItem(4,  QTableWidgetItem(self.tr( "Total" )))

        table.applySettings()
        (sum_io_retentions, sum_div_retentions, sum_other_taxes,  sum_returned_taxes)=(Decimal("0"), Decimal("0"), Decimal("0"), Decimal("0"))

        for column in range (12):
            io_retentions=-none2decimal0(self.mem.con.cursor_one_row("""select sum(impuestos) 
                                                                                                                                from operinversiones  
                                                                                                                                where date_part('year',datetime)=%s and 
                                                                                                                                    date_part('month',datetime)=%s;""", (self.wyData.year, column+1))[0])
            table.setItem(0, column, self.mem.localcurrency.qtablewidgetitem(io_retentions))    
            sum_io_retentions=sum_io_retentions+io_retentions
            
            div_retentions=-none2decimal0(self.mem.con.cursor_one_row("""select sum(retencion) 
                                                                                                            from dividends 
                                                                                                            where date_part('year',fecha)=%s and 
                                                                                                                date_part('month',fecha)=%s;""", (self.wyData.year, column+1))[0])
            table.setItem(1, column, self.mem.localcurrency.qtablewidgetitem(div_retentions))    
            sum_div_retentions=sum_div_retentions+div_retentions
            
            other_taxes=none2decimal0(self.mem.con.cursor_one_row("""select sum(importe) 
                                                                                                            from opercuentas 
                                                                                                            where id_conceptos=%s and 
                                                                                                                date_part('year',datetime)=%s and 
                                                                                                                date_part('month',datetime)=%s;""", (37, self.wyData.year, column+1))[0])
            table.setItem(2, column, self.mem.localcurrency.qtablewidgetitem(other_taxes))    
            sum_other_taxes=sum_other_taxes+other_taxes         
            
            returned_taxes=none2decimal0(self.mem.con.cursor_one_row("""select sum(importe) 
                                                                                                            from opercuentas 
                                                                                                            where id_conceptos=%s and 
                                                                                                                date_part('year',datetime)=%s and 
                                                                                                                date_part('month',datetime)=%s;""", (6, self.wyData.year, column+1))[0])
            table.setItem(3, column, self.mem.localcurrency.qtablewidgetitem(returned_taxes))    
            sum_returned_taxes=sum_returned_taxes+returned_taxes
            
            table.setItem(4, column, self.mem.localcurrency.qtablewidgetitem(io_retentions+div_retentions+other_taxes+returned_taxes))    
        
        table.setItem(0, 12, self.mem.localcurrency.qtablewidgetitem(sum_io_retentions))    
        table.setItem(1, 12, self.mem.localcurrency.qtablewidgetitem(sum_div_retentions))    
        table.setItem(2, 12, self.mem.localcurrency.qtablewidgetitem(sum_other_taxes))    
        table.setItem(3, 12, self.mem.localcurrency.qtablewidgetitem(sum_returned_taxes))    
        table.setItem(4, 12, self.mem.localcurrency.qtablewidgetitem(sum_io_retentions+sum_div_retentions+sum_other_taxes+sum_returned_taxes))    

        horizontalLayout.addWidget(table)
        self.tab.addTab(newtab, self.tr("Taxes report of {}").format(self.wyData.year))
        self.tab.setCurrentWidget(newtab)        
            

    def on_tab_tabCloseRequested(self, index):
        """Only removes dinamic tabs"""
        if index in (0, 1):
            qmessagebox(self.tr("You can't close this tab"))
        else:
            self.tab.setCurrentIndex(0)
            self.tab.removeTab(index)
            
    def on_table_cellDoubleClicked(self, row, column):
        if row==0:#incomes
            self.on_actionShowIncomes_triggered()
        elif row==1:#Gains
            self.on_actionShowSellingOperations_triggered()
        elif row==2:#Dividends
            self.on_actionShowDividends_triggered()
        elif row==3: #Expenses
            self.on_actionShowExpenses_triggered()
        elif row==7: #Investments
            totalmonth=self.setData.arr[column]
            qmessagebox(self.tr("High Low Investments aren't sumarized here, due to they have daily adjustments in accounts.") + "\n\n" + self.tr("Their balance at the end of {}-{} is {}").format(totalmonth.year, totalmonth.month, totalmonth.total_investments_high_low()))
        else:
            qmessagebox(self.tr("You only can double click in incomes, gains, dividends and expenses.") + "\n\n" + self.tr("Make right click to see comission and tax reports"))

    def on_table_customContextMenuRequested(self,  pos):
        menu=QMenu()
        menu.addAction(self.actionShowComissions)
        menu.addSeparator()
        menu.addAction(self.actionShowTaxes)
        menu.addSeparator()
        menu.addAction(self.actionGainsByProductType)
        menu.exec_(self.table.mapToGlobal(pos))

    def on_table_itemSelectionChanged(self):
        self.month=None
        for i in self.table.selectedItems():#itera por cada item no row.
            self.month=i.column()+1
        print ("Selected month: {0}.".format(self.month))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'xulpymoney/ui/wdgTotal.ui'
#
# Created by: PyQt5 UI code generator 5.12.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_wdgTotal(object):
    def setupUi(self, wdgTotal):
        wdgTotal.setObjectName("wdgTotal")
        wdgTotal.resize(1512, 815)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(wdgTotal)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.lblTitulo = QtWidgets.QLabel(wdgTotal)
        font = QtGui.QFont()
        font.setPointSize(14)
        font.setBold(True)
        font.setWeight(75)
        self.lblTitulo.setFont(font)
        self.lblTitulo.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTitulo.setObjectName("lblTitulo")
        self.verticalLayout_2.addWidget(self.lblTitulo)
        self.tab = QtWidgets.QTabWidget(wdgTotal)
        self.tab.setTabsClosable(True)
        self.tab.setObjectName("tab")
        self.tabDatos = QtWidgets.QWidget()
        self.tabDatos.setObjectName("tabDatos")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.tabDatos)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout()
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.wyData = wdgYear(self.tabDatos)
        self.wyData.setObjectName("wyData")
        self.horizontalLayout.addWidget(self.wyData)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout_7.addLayout(self.horizontalLayout)
        self.tabData = QtWidgets.QTabWidget(self.tabDatos)
        self.tabData.setTabPosition(QtWidgets.QTabWidget.West)
        self.tabData.setObjectName("tabData")
        self.tab_3 = QtWidgets.QWidget()
        self.tab_3.setObjectName("tab_3")
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout(self.tab_3)
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.lblPreviousYear = QtWidgets.QLabel(self.tab_3)
        font = QtGui.QFont()
        font.setPointSize(8)
        font.setBold(True)
        font.setWeight(75)
        self.lblPreviousYear.setFont(font)
        self.lblPreviousYear.setAlignment(QtCore.Qt.AlignCenter)
        self.lblPreviousYear.setObjectName("lblPreviousYear")
        self.verticalLayout.addWidget(self.lblPreviousYear)
        self.table = myQTableWidget(self.tab_3)
        self.table.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.table.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.table.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.table.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectItems)
        self.table.setObjectName("table")
        self.table.setColumnCount(13)
        self.table.setRowCount(12)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(8, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(9, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(10, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setVerticalHeaderItem(11, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(8, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(9, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(10, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(11, item)
        item = QtWidgets.QTableWidgetItem()
        self.table.setHorizontalHeaderItem(12, item)
        self.verticalLayout.addWidget(self.table)
        self.lblInvested = QtWidgets.QLabel(self.tab_3)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lblInvested.setFont(font)
        self.lblInvested.setText("")
        self.lblInvested.setAlignment(QtCore.Qt.AlignCenter)
        self.lblInvested.setObjectName("lblInvested")
        self.verticalLayout.addWidget(self.lblInvested)
        self.horizontalLayout_8.addLayout(self.verticalLayout)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/xulpymoney/coins.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabData.addTab(self.tab_3, icon, "")
        self.tab_4 = QtWidgets.QWidget()
        self.tab_4.setObjectName("tab_4")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.tab_4)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem2)
        self.lblTarget = QtWidgets.QLabel(self.tab_4)
        self.lblTarget.setObjectName("lblTarget")
        self.horizontalLayout_5.addWidget(self.lblTarget)
        self.spinTarget = QtWidgets.QDoubleSpinBox(self.tab_4)
        self.spinTarget.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.spinTarget.setObjectName("spinTarget")
        self.horizontalLayout_5.addWidget(self.spinTarget)
        self.cmdTargets = QtWidgets.QToolButton(self.tab_4)
        self.cmdTargets.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/xulpymoney/save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cmdTargets.setIcon(icon1)
        self.cmdTargets.setObjectName("cmdTargets")
        self.horizontalLayout_5.addWidget(self.cmdTargets)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem3)
        self.verticalLayout_8.addLayout(self.horizontalLayout_5)
        self.tabPlus = QtWidgets.QTabWidget(self.tab_4)
        self.tabPlus.setObjectName("tabPlus")
        self.tab_6 = QtWidgets.QWidget()
        self.tab_6.setObjectName("tab_6")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout(self.tab_6)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.tblTargets = myQTableWidget(self.tab_6)
        self.tblTargets.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblTargets.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblTargets.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblTargets.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectItems)
        self.tblTargets.setObjectName("tblTargets")
        self.tblTargets.setColumnCount(13)
        self.tblTargets.setRowCount(5)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setVerticalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setVerticalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setVerticalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setVerticalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setVerticalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(8, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(9, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(10, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(11, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargets.setHorizontalHeaderItem(12, item)
        self.verticalLayout_3.addWidget(self.tblTargets)
        self.lblTargets = QtWidgets.QLabel(self.tab_6)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lblTargets.setFont(font)
        self.lblTargets.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTargets.setObjectName("lblTargets")
        self.verticalLayout_3.addWidget(self.lblTargets)
        self.horizontalLayout_10.addLayout(self.verticalLayout_3)
        self.tabPlus.addTab(self.tab_6, "")
        self.tab_7 = QtWidgets.QWidget()
        self.tab_7.setObjectName("tab_7")
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout(self.tab_7)
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.tblTargetsPlus = myQTableWidget(self.tab_7)
        self.tblTargetsPlus.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblTargetsPlus.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblTargetsPlus.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblTargetsPlus.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectItems)
        self.tblTargetsPlus.setObjectName("tblTargetsPlus")
        self.tblTargetsPlus.setColumnCount(13)
        self.tblTargetsPlus.setRowCount(7)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setVerticalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setVerticalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setVerticalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setVerticalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setVerticalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setVerticalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setVerticalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(8, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(9, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(10, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(11, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblTargetsPlus.setHorizontalHeaderItem(12, item)
        self.verticalLayout_9.addWidget(self.tblTargetsPlus)
        self.lblTargetsPlus = QtWidgets.QLabel(self.tab_7)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lblTargetsPlus.setFont(font)
        self.lblTargetsPlus.setAlignment(QtCore.Qt.AlignCenter)
        self.lblTargetsPlus.setObjectName("lblTargetsPlus")
        self.verticalLayout_9.addWidget(self.lblTargetsPlus)
        self.horizontalLayout_11.addLayout(self.verticalLayout_9)
        self.tabPlus.addTab(self.tab_7, "")
        self.verticalLayout_8.addWidget(self.tabPlus)
        self.horizontalLayout_4.addLayout(self.verticalLayout_8)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/xulpymoney/gafas.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabData.addTab(self.tab_4, icon2, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.tab_2)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.tblInvestOrWork = myQTableWidget(self.tab_2)
        self.tblInvestOrWork.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblInvestOrWork.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblInvestOrWork.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblInvestOrWork.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectItems)
        self.tblInvestOrWork.setObjectName("tblInvestOrWork")
        self.tblInvestOrWork.setColumnCount(13)
        self.tblInvestOrWork.setRowCount(6)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setVerticalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setVerticalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setVerticalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setVerticalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setVerticalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setVerticalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(8, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(9, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(10, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(11, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblInvestOrWork.setHorizontalHeaderItem(12, item)
        self.verticalLayout_5.addWidget(self.tblInvestOrWork)
        self.lblInvestOrWork = QtWidgets.QLabel(self.tab_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.lblInvestOrWork.setFont(font)
        self.lblInvestOrWork.setText("")
        self.lblInvestOrWork.setAlignment(QtCore.Qt.AlignCenter)
        self.lblInvestOrWork.setObjectName("lblInvestOrWork")
        self.verticalLayout_5.addWidget(self.lblInvestOrWork)
        self.horizontalLayout_7.addLayout(self.verticalLayout_5)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/xulpymoney/tools-wizard.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabData.addTab(self.tab_2, icon3, "")
        self.tab_5 = QtWidgets.QWidget()
        self.tab_5.setObjectName("tab_5")
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout(self.tab_5)
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.tblMakeEndsMeet = myQTableWidget(self.tab_5)
        self.tblMakeEndsMeet.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tblMakeEndsMeet.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tblMakeEndsMeet.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tblMakeEndsMeet.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectItems)
        self.tblMakeEndsMeet.setObjectName("tblMakeEndsMeet")
        self.tblMakeEndsMeet.setColumnCount(13)
        self.tblMakeEndsMeet.setRowCount(6)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setVerticalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setVerticalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setVerticalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setVerticalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setVerticalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setVerticalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(3, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(4, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(5, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(6, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(7, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(8, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(9, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(10, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(11, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblMakeEndsMeet.setHorizontalHeaderItem(12, item)
        self.verticalLayout_6.addWidget(self.tblMakeEndsMeet)
        self.lblMakeEndsMeet = QtWidgets.QLabel(self.tab_5)
        self.lblMakeEndsMeet.setText("")
        self.lblMakeEndsMeet.setObjectName("lblMakeEndsMeet")
        self.verticalLayout_6.addWidget(self.lblMakeEndsMeet)
        self.horizontalLayout_9.addLayout(self.verticalLayout_6)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/xulpymoney/books.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tabData.addTab(self.tab_5, icon4, "")
        self.verticalLayout_7.addWidget(self.tabData)
        self.horizontalLayout_3.addLayout(self.verticalLayout_7)
        self.tab.addTab(self.tabDatos, icon, "")
        self.tabGraphic = QtWidgets.QWidget()
        self.tabGraphic.setObjectName("tabGraphic")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tabGraphic)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.tabGraphTotal = QtWidgets.QVBoxLayout()
        self.tabGraphTotal.setObjectName("tabGraphTotal")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem4)
        self.wyChart = wdgYear(self.tabGraphic)
        self.wyChart.setObjectName("wyChart")
        self.horizontalLayout_6.addWidget(self.wyChart)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem5)
        self.tabGraphTotal.addLayout(self.horizontalLayout_6)
        self.verticalLayout_4.addLayout(self.tabGraphTotal)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/xulpymoney/pie.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tab.addTab(self.tabGraphic, icon5, "")
        self.verticalLayout_2.addWidget(self.tab)
        self.horizontalLayout_2.addLayout(self.verticalLayout_2)
        self.actionShowIncomes = QtWidgets.QAction(wdgTotal)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/xulpymoney/bundle.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowIncomes.setIcon(icon6)
        self.actionShowIncomes.setObjectName("actionShowIncomes")
        self.actionShowExpenses = QtWidgets.QAction(wdgTotal)
        self.actionShowExpenses.setObjectName("actionShowExpenses")
        self.actionShowDividends = QtWidgets.QAction(wdgTotal)
        self.actionShowDividends.setIcon(icon)
        self.actionShowDividends.setObjectName("actionShowDividends")
        self.actionShowSellingOperations = QtWidgets.QAction(wdgTotal)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/xulpymoney/dinero.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowSellingOperations.setIcon(icon7)
        self.actionShowSellingOperations.setObjectName("actionShowSellingOperations")
        self.actionShowComissions = QtWidgets.QAction(wdgTotal)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/xulpymoney/bank.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowComissions.setIcon(icon8)
        self.actionShowComissions.setObjectName("actionShowComissions")
        self.actionShowTaxes = QtWidgets.QAction(wdgTotal)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/xulpymoney/study.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionShowTaxes.setIcon(icon9)
        self.actionShowTaxes.setObjectName("actionShowTaxes")
        self.actionGainsByProductType = QtWidgets.QAction(wdgTotal)
        self.actionGainsByProductType.setIcon(icon)
        self.actionGainsByProductType.setObjectName("actionGainsByProductType")

        self.retranslateUi(wdgTotal)
        self.tab.setCurrentIndex(0)
        self.tabData.setCurrentIndex(1)
        self.tabPlus.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(wdgTotal)

    def retranslateUi(self, wdgTotal):
        _translate = QtCore.QCoreApplication.translate
        self.lblTitulo.setText(_translate("wdgTotal", "Total report"))
        self.table.setSortingEnabled(False)
        item = self.table.verticalHeaderItem(0)
        item.setText(_translate("wdgTotal", "Incomes"))
        item = self.table.verticalHeaderItem(1)
        item.setText(_translate("wdgTotal", "Gains"))
        item = self.table.verticalHeaderItem(2)
        item.setText(_translate("wdgTotal", "Dividends"))
        item = self.table.verticalHeaderItem(3)
        item.setText(_translate("wdgTotal", "Expenses"))
        item = self.table.verticalHeaderItem(4)
        item.setText(_translate("wdgTotal", "I+G+D-E"))
        item = self.table.verticalHeaderItem(6)
        item.setText(_translate("wdgTotal", "Accounts"))
        item = self.table.verticalHeaderItem(7)
        item.setText(_translate("wdgTotal", "Investments"))
        item = self.table.verticalHeaderItem(8)
        item.setText(_translate("wdgTotal", "Total"))
        item = self.table.verticalHeaderItem(9)
        item.setText(_translate("wdgTotal", "Monthly difference"))
        item = self.table.verticalHeaderItem(11)
        item.setText(_translate("wdgTotal", "% Year to date"))
        item = self.table.horizontalHeaderItem(0)
        item.setText(_translate("wdgTotal", "January"))
        item = self.table.horizontalHeaderItem(1)
        item.setText(_translate("wdgTotal", "February"))
        item = self.table.horizontalHeaderItem(2)
        item.setText(_translate("wdgTotal", "March"))
        item = self.table.horizontalHeaderItem(3)
        item.setText(_translate("wdgTotal", "April"))
        item = self.table.horizontalHeaderItem(4)
        item.setText(_translate("wdgTotal", "May"))
        item = self.table.horizontalHeaderItem(5)
        item.setText(_translate("wdgTotal", "June"))
        item = self.table.horizontalHeaderItem(6)
        item.setText(_translate("wdgTotal", "July"))
        item = self.table.horizontalHeaderItem(7)
        item.setText(_translate("wdgTotal", "August"))
        item = self.table.horizontalHeaderItem(8)
        item.setText(_translate("wdgTotal", "September"))
        item = self.table.horizontalHeaderItem(9)
        item.setText(_translate("wdgTotal", "October"))
        item = self.table.horizontalHeaderItem(10)
        item.setText(_translate("wdgTotal", "November"))
        item = self.table.horizontalHeaderItem(11)
        item.setText(_translate("wdgTotal", "December"))
        item = self.table.horizontalHeaderItem(12)
        item.setText(_translate("wdgTotal", "Total"))
        self.tabData.setTabText(self.tabData.indexOf(self.tab_3), _translate("wdgTotal", "Data in the year"))
        self.lblTarget.setText(_translate("wdgTotal", "Annual target percentage"))
        self.spinTarget.setSuffix(_translate("wdgTotal", " %"))
        self.tblTargets.setSortingEnabled(False)
        item = self.tblTargets.verticalHeaderItem(0)
        item.setText(_translate("wdgTotal", "Monthly target"))
        item = self.tblTargets.verticalHeaderItem(1)
        item.setText(_translate("wdgTotal", "Total Gains"))
        item = self.tblTargets.verticalHeaderItem(3)
        item.setText(_translate("wdgTotal", "Accumulated target"))
        item = self.tblTargets.verticalHeaderItem(4)
        item.setText(_translate("wdgTotal", "Accumulated total gains"))
        item = self.tblTargets.horizontalHeaderItem(0)
        item.setText(_translate("wdgTotal", "January"))
        item = self.tblTargets.horizontalHeaderItem(1)
        item.setText(_translate("wdgTotal", "February"))
        item = self.tblTargets.horizontalHeaderItem(2)
        item.setText(_translate("wdgTotal", "March"))
        item = self.tblTargets.horizontalHeaderItem(3)
        item.setText(_translate("wdgTotal", "April"))
        item = self.tblTargets.horizontalHeaderItem(4)
        item.setText(_translate("wdgTotal", "May"))
        item = self.tblTargets.horizontalHeaderItem(5)
        item.setText(_translate("wdgTotal", "June"))
        item = self.tblTargets.horizontalHeaderItem(6)
        item.setText(_translate("wdgTotal", "July"))
        item = self.tblTargets.horizontalHeaderItem(7)
        item.setText(_translate("wdgTotal", "August"))
        item = self.tblTargets.horizontalHeaderItem(8)
        item.setText(_translate("wdgTotal", "September"))
        item = self.tblTargets.horizontalHeaderItem(9)
        item.setText(_translate("wdgTotal", "October"))
        item = self.tblTargets.horizontalHeaderItem(10)
        item.setText(_translate("wdgTotal", "November"))
        item = self.tblTargets.horizontalHeaderItem(11)
        item.setText(_translate("wdgTotal", "December"))
        item = self.tblTargets.horizontalHeaderItem(12)
        item.setText(_translate("wdgTotal", "Total"))
        self.tabPlus.setTabText(self.tabPlus.indexOf(self.tab_6), _translate("wdgTotal", "Consolidated Gains"))
        self.tblTargetsPlus.setSortingEnabled(False)
        item = self.tblTargetsPlus.verticalHeaderItem(0)
        item.setText(_translate("wdgTotal", "Monthly target"))
        item = self.tblTargetsPlus.verticalHeaderItem(1)
        item.setText(_translate("wdgTotal", "Total Gains"))
        item = self.tblTargetsPlus.verticalHeaderItem(2)
        item.setText(_translate("wdgTotal", "Funds Revaluation"))
        item = self.tblTargetsPlus.verticalHeaderItem(3)
        item.setText(_translate("wdgTotal", "Total"))
        item = self.tblTargetsPlus.verticalHeaderItem(5)
        item.setText(_translate("wdgTotal", "Accumulated target"))
        item = self.tblTargetsPlus.verticalHeaderItem(6)
        item.setText(_translate("wdgTotal", "Accumulated total gains"))
        item = self.tblTargetsPlus.horizontalHeaderItem(0)
        item.setText(_translate("wdgTotal", "January"))
        item = self.tblTargetsPlus.horizontalHeaderItem(1)
        item.setText(_translate("wdgTotal", "February"))
        item = self.tblTargetsPlus.horizontalHeaderItem(2)
        item.setText(_translate("wdgTotal", "March"))
        item = self.tblTargetsPlus.horizontalHeaderItem(3)
        item.setText(_translate("wdgTotal", "April"))
        item = self.tblTargetsPlus.horizontalHeaderItem(4)
        item.setText(_translate("wdgTotal", "May"))
        item = self.tblTargetsPlus.horizontalHeaderItem(5)
        item.setText(_translate("wdgTotal", "June"))
        item = self.tblTargetsPlus.horizontalHeaderItem(6)
        item.setText(_translate("wdgTotal", "July"))
        item = self.tblTargetsPlus.horizontalHeaderItem(7)
        item.setText(_translate("wdgTotal", "August"))
        item = self.tblTargetsPlus.horizontalHeaderItem(8)
        item.setText(_translate("wdgTotal", "September"))
        item = self.tblTargetsPlus.horizontalHeaderItem(9)
        item.setText(_translate("wdgTotal", "October"))
        item = self.tblTargetsPlus.horizontalHeaderItem(10)
        item.setText(_translate("wdgTotal", "November"))
        item = self.tblTargetsPlus.horizontalHeaderItem(11)
        item.setText(_translate("wdgTotal", "December"))
        item = self.tblTargetsPlus.horizontalHeaderItem(12)
        item.setText(_translate("wdgTotal", "Total"))
        self.tabPlus.setTabText(self.tabPlus.indexOf(self.tab_7), _translate("wdgTotal", "Consolidated Gains + funds revaluation"))
        self.tabData.setTabText(self.tabData.indexOf(self.tab_4), _translate("wdgTotal", "Annual target return"))
        self.tblInvestOrWork.setSortingEnabled(False)
        item = self.tblInvestOrWork.verticalHeaderItem(0)
        item.setText(_translate("wdgTotal", "Total Gains"))
        item = self.tblInvestOrWork.verticalHeaderItem(1)
        item.setText(_translate("wdgTotal", "Expenses"))
        item = self.tblInvestOrWork.verticalHeaderItem(3)
        item.setText(_translate("wdgTotal", "Total Gains - Expenses"))
        item = self.tblInvestOrWork.verticalHeaderItem(5)
        item.setText(_translate("wdgTotal", "Result"))
        item = self.tblInvestOrWork.horizontalHeaderItem(0)
        item.setText(_translate("wdgTotal", "January"))
        item = self.tblInvestOrWork.horizontalHeaderItem(1)
        item.setText(_translate("wdgTotal", "February"))
        item = self.tblInvestOrWork.horizontalHeaderItem(2)
        item.setText(_translate("wdgTotal", "March"))
        item = self.tblInvestOrWork.horizontalHeaderItem(3)
        item.setText(_translate("wdgTotal", "April"))
        item = self.tblInvestOrWork.horizontalHeaderItem(4)
        item.setText(_translate("wdgTotal", "May"))
        item = self.tblInvestOrWork.horizontalHeaderItem(5)
        item.setText(_translate("wdgTotal", "June"))
        item = self.tblInvestOrWork.horizontalHeaderItem(6)
        item.setText(_translate("wdgTotal", "July"))
        item = self.tblInvestOrWork.horizontalHeaderItem(7)
        item.setText(_translate("wdgTotal", "August"))
        item = self.tblInvestOrWork.horizontalHeaderItem(8)
        item.setText(_translate("wdgTotal", "September"))
        item = self.tblInvestOrWork.horizontalHeaderItem(9)
        item.setText(_translate("wdgTotal", "October"))
        item = self.tblInvestOrWork.horizontalHeaderItem(10)
        item.setText(_translate("wdgTotal", "November"))
        item = self.tblInvestOrWork.horizontalHeaderItem(11)
        item.setText(_translate("wdgTotal", "December"))
        item = self.tblInvestOrWork.horizontalHeaderItem(12)
        item.setText(_translate("wdgTotal", "Total"))
        self.tabData.setTabText(self.tabData.indexOf(self.tab_2), _translate("wdgTotal", "Invest or work?"))
        self.tblMakeEndsMeet.setSortingEnabled(False)
        item = self.tblMakeEndsMeet.verticalHeaderItem(0)
        item.setText(_translate("wdgTotal", "Incomes"))
        item = self.tblMakeEndsMeet.verticalHeaderItem(1)
        item.setText(_translate("wdgTotal", "Expenses"))
        item = self.tblMakeEndsMeet.verticalHeaderItem(3)
        item.setText(_translate("wdgTotal", "Incomes - Expenses"))
        item = self.tblMakeEndsMeet.verticalHeaderItem(5)
        item.setText(_translate("wdgTotal", "Result"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(0)
        item.setText(_translate("wdgTotal", "January"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(1)
        item.setText(_translate("wdgTotal", "February"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(2)
        item.setText(_translate("wdgTotal", "March"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(3)
        item.setText(_translate("wdgTotal", "April"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(4)
        item.setText(_translate("wdgTotal", "May"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(5)
        item.setText(_translate("wdgTotal", "June"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(6)
        item.setText(_translate("wdgTotal", "July"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(7)
        item.setText(_translate("wdgTotal", "August"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(8)
        item.setText(_translate("wdgTotal", "September"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(9)
        item.setText(_translate("wdgTotal", "October"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(10)
        item.setText(_translate("wdgTotal", "November"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(11)
        item.setText(_translate("wdgTotal", "December"))
        item = self.tblMakeEndsMeet.horizontalHeaderItem(12)
        item.setText(_translate("wdgTotal", "Total"))
        self.tabData.setTabText(self.tabData.indexOf(self.tab_5), _translate("wdgTotal", "Make ends meet?"))
        self.tab.setTabText(self.tab.indexOf(self.tabDatos), _translate("wdgTotal", "Data"))
        self.tab.setTabText(self.tab.indexOf(self.tabGraphic), _translate("wdgTotal", "Chart"))
        self.actionShowIncomes.setText(_translate("wdgTotal", "Show income operations"))
        self.actionShowIncomes.setToolTip(_translate("wdgTotal", "Show income operations"))
        self.actionShowExpenses.setText(_translate("wdgTotal", "Show expense operations"))
        self.actionShowDividends.setText(_translate("wdgTotal", "Show dividends"))
        self.actionShowDividends.setToolTip(_translate("wdgTotal", "Show dividends"))
        self.actionShowSellingOperations.setText(_translate("wdgTotal", "Show selling operations"))
        self.actionShowSellingOperations.setToolTip(_translate("wdgTotal", "Show selling operations"))
        self.actionShowComissions.setText(_translate("wdgTotal", "Show Comissions"))
        self.actionShowComissions.setToolTip(_translate("wdgTotal", "Show Comissions"))
        self.actionShowTaxes.setText(_translate("wdgTotal", "Show taxes"))
        self.actionShowTaxes.setToolTip(_translate("wdgTotal", "Show taxes"))
        self.actionGainsByProductType.setText(_translate("wdgTotal", "Gains by product type"))
        self.actionGainsByProductType.setToolTip(_translate("wdgTotal", "Gains by product type"))


from xulpymoney.ui.myqtablewidget import myQTableWidget
from xulpymoney.ui.wdgYear import wdgYear
import xulpymoney.images.xulpymoney_rc



from PyQt5.QtCore import pyqtSlot
from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import QDialog
from xulpymoney.ui.Ui_frmAccess import Ui_frmAccess
from xulpymoney.connection_pg_qt import ConnectionQt
from xulpymoney.libxulpymoneyfunctions import qmessagebox

class frmAccess(QDialog, Ui_frmAccess):
    def __init__(self, mem, parent = None, name = None, modal = False):
        """Returns accepted if conection is done, or rejected if there's an error"""""
        QDialog.__init__(self,  parent)
        self.mem=mem
        self.setModal(True)
        self.setupUi(self)
        self.parent=parent
        self.cmbLanguages.disconnect()
        self.mem.languages.qcombobox(self.cmbLanguages, self.mem.language)
        self.cmbLanguages.currentIndexChanged.connect(self.on_cmbLanguages_currentIndexChanged)
        self.setPixmap(QPixmap(":xulpymoney/coins.png"))
        self.setTitle(self.tr("Xulpymoney - Access"))
        self.con=ConnectionQt()#Pointer to connection


    def setPixmap(self, qpixmap):
        icon = QIcon()
        icon.addPixmap(qpixmap, QIcon.Normal, QIcon.Off)
        self.setWindowIcon(icon)        
        
    def setTitle(self, text):
        self.setWindowTitle(text)
        
    def setLabel(self, text):
        self.lbl.setText(text)
        
    def showLanguage(self, boolean):
        if boolean==False:
            self.cmbLanguages.hide()
            self.lblLanguage.hide()
        
        
    def config_load(self):
        self.txtDB.setText(self.mem.settings.value("frmAccess/db", "xulpymoney" ))
        self.txtPort.setText(self.mem.settings.value("frmAccess/port", "5432"))
        self.txtUser.setText(self.mem.settings.value("frmAccess/user", "postgres" ))
        self.txtServer.setText(self.mem.settings.value("frmAccess/server", "127.0.0.1" ))
        self.txtPass.setFocus()
        
    def config_save(self):
        self.mem.settings.setValue("frmAccess/db", self.txtDB.text() )
        self.mem.settings.setValue("frmAccess/port",  self.txtPort.text())
        self.mem.settings.setValue("frmAccess/user" ,  self.txtUser.text())
        self.mem.settings.setValue("frmAccess/server", self.txtServer.text())   
        self.mem.settings.setValue("mem/language", self.cmbLanguages.itemData(self.cmbLanguages.currentIndex()))
        self.mem.language=self.mem.languages.find_by_id(self.cmbLanguages.itemData(self.cmbLanguages.currentIndex()))

    @pyqtSlot(int)      
    def on_cmbLanguages_currentIndexChanged(self, stri):
        self.mem.language=self.mem.languages.find_by_id(self.cmbLanguages.itemData(self.cmbLanguages.currentIndex()))
        self.mem.settings.setValue("mem/language", self.mem.language.id)
        self.mem.languages.cambiar(self.mem.language.id)
        self.retranslateUi(self)

    def make_connection(self):
        """Función que realiza la conexión devolviendo true o false con el éxito"""
        try:
            self.con.init__create(self.txtUser.text(), self.txtPass.text(), self.txtServer.text(), self.txtPort.text(), self.txtDB.text())
            self.con.connect()
            return self.con.is_active()
        except:
            print ("Error in function make_connection",  self.mem.con)
            return False
    
    @pyqtSlot() 
    def on_cmdYN_accepted(self):
        self.con.init__create(self.txtUser.text(), self.txtPass.text(), self.txtServer.text(), self.txtPort.text(), self.txtDB.text())
        self.con.connect()
        if self.con.is_active():
            self.config_save()
            self.accept()
        else:
            qmessagebox(self.tr("Error conecting to {} database in {} server").format(self.con.db, self.con.server))
            self.reject()

    @pyqtSlot() 
    def on_cmdYN_rejected(self):
        self.reject()






## @namespace xulpymoney.libxulpymoneytypes
## @brief Package with all xulpymoney types.
from PyQt5.QtGui import QColor, QIcon
from PyQt5.QtWidgets import QApplication
from enum import IntEnum

## Class with used QColors in app
class eQColor:
    Red=QColor(255, 148, 148)
    Green=QColor(148, 255, 148)
        
class eProductType(IntEnum):
    """
        IntEnum permite comparar 1 to eProductType.Share
    """
    Share=1
    Fund=2
    Index=3
    ETF=4
    Warrant=5
    Currency=6
    PublicBond=7
    PensionPlan=8
    PrivateBond=9
    Deposit=10
    Account=11
    CFD=12
    Future=13
    
## Operation tipes
class eOperationType:
    Expense=1
    Income=2
    Transfer=3
    SharesPurchase=4
    SharesSale=5
    SharesAdd=6
    CreditCardBilling=7
    TransferFunds=8
    TransferSharesOrigin=9
    TransferSharesDestiny=10
    HlContract=11
    
class eTickerPosition(IntEnum):
    """It's the number to access to a python list,  not to postgresql. In postgres it will be +1"""
    Yahoo=0
    Morningstar=1
    Google=2
    QueFondos=3
    
    def postgresql(etickerposition):
        return etickerposition.value+1
        
    ## Returns the number of atributes
    def length():
        return len(eTickerPosition.__dict__)


class eComment:
    InvestmentOperation=10000
    Dividend=10004
    HlContract=10007
    AccountTransferOrigin=10001
    AccountTransferDestiny=10002
    AccountTransferOriginCommission=10003
    CreditCardBilling=10005
    CreditCardRefund=10006# Estaba en 10007


## System concepts tipified
class eConcept:
    OpenAccount=1
    TransferOrigin=4
    TransferDestiny=5
    TaxesReturn=6
    BuyShares=29
    SellShares=35
    TaxesPayment=37
    BankCommissions=38
    Dividends=39
    CreditCardBilling=40
    AddShares=43
    AssistancePremium=50
    CommissionCustody=59
    DividendsSaleRights=62
    BondsCouponRunPayment=63
    BondsCouponRunIncome=65
    BondsCoupon=66
    CreditCardRefund=67
    HlAdjustmentIincome=68
    HlAdjustmentExpense=69
    HlGuaranteePaid=70
    HlGuaranteeReturned=71
    HlCommission=72
    HlInterestPaid=73
    HlInterestReceived=74
    RolloverPaid=75
    RolloverReceived=76

## Types for dt strings. Used in dtaware2string function
class eDtStrings:
    QTableWidgetItem=1
    Filename=2

## Sets if a Historical Chart must adjust splits or dividends with splits or do nothing
class eHistoricalChartAdjusts:
    ## Without splits nor dividens
    NoAdjusts=0
    ## WithSplits
    Splits=1
    ##With splits and dividends
    SplitsAndDividends=2#Dividends with splits.        
    
class eOHCLDuration:
    Day=1
    Week=2
    Month=3
    Year=4

    @classmethod
    def qcombobox(self, combo, selected_eOHCLDuration):
        combo.addItem(QApplication.translate("Core", "Day"), 1)
        combo.addItem(QApplication.translate("Core", "Week"), 2)
        combo.addItem(QApplication.translate("Core", "Month"), 3)
        combo.addItem(QApplication.translate("Core", "Year"), 4)
        combo.setCurrentIndex(combo.findData(selected_eOHCLDuration))

class eLeverageType:
    Variable=-1
    NotLeveraged=1
    X2=2
    X3=3
    X4=4
    X5=5
    X10=10
    X50=50
    X100=100

class eMoneyCurrency:
    Product=1
    Account=2
    User=3

## Type definition to refer to long /short invesment type positions
class eInvestmentTypePosition:
    Long=1
    Short=2

    @classmethod
    def qicon_boolean(self, boolean):
        e=eInvestmentTypePosition.to_eInvestmentTypePosition(boolean)
        return eInvestmentTypePosition.qicon(e)

    @classmethod
    def qicon(self, einvestmenttypeposition):
        if einvestmenttypeposition==eInvestmentTypePosition.Long:
            return QIcon(":/xulpymoney/up.png")
        else:
            return QIcon(":/xulpymoney/down.png")
            

    @classmethod
    def qcombobox(self, combo, selected_eInvestmentTypePosition):
        combo.addItem(eInvestmentTypePosition.qicon(eInvestmentTypePosition.Long), QApplication.translate("Core", "Long"), eInvestmentTypePosition.Long)
        combo.addItem(eInvestmentTypePosition.qicon(eInvestmentTypePosition.Short), QApplication.translate("Core", "Short"), eInvestmentTypePosition.Short)
        combo.setCurrentIndex(combo.findData(selected_eInvestmentTypePosition))

    ## Return True if it's short. Due to postgres database has this definition
    @classmethod
    def to_boolean(self, einvestmenttypeposition):
        if einvestmenttypeposition==1:
            return False
        return True

    ## Returns Short if boolean is true
    @classmethod
    def to_eInvestmentTypePosition(self, boolean):
        if boolean==True:
            return eInvestmentTypePosition.Short
        return eInvestmentTypePosition.Long



from PyQt5.QtWidgets import QWidget, QTableWidgetItem,  QMessageBox
from xulpymoney.ui.Ui_wdgProductsDataMove import Ui_wdgProductsDataMove
from xulpymoney.libxulpymoneyfunctions import qcenter, qright

class wdgProductsDataMove(QWidget, Ui_wdgProductsDataMove):
    def __init__(self, mem,  origin, destiny, parent = None, name = None, modal = False):
        QWidget.__init__(self,  parent)
        self.mem=mem
        self.origin=origin
        self.destiny=destiny
        self.setupUi(self)
        self.table.settings(self.mem, "wdgProductsDataMove") 
        self.origin.needStatus(3)
        self.destiny.needStatus(3)
        self.reload()
        
    def on_cmdInterchange_released(self):
        tmp=self.origin
        self.origin=self.destiny
        self.destiny=tmp
        self.reload()
    
    ## Sets tabble data
    def reload(self):
        self.table.applySettings()
        for i,  p in enumerate([self.origin, self.destiny]):
            self.table.setItem(i, 0, qcenter(p.id))
            self.table.item(i, 0).setIcon(p.stockmarket.country.qicon())
            self.table.setItem(i, 1, QTableWidgetItem(p.name))
            self.table.setItem(i, 2, QTableWidgetItem(p.isin))
            self.table.setItem(i, 3, qright(p.result.all.length()))
            self.table.setItem(i, 4, qright(p.dps.length()))
            self.table.setItem(i, 5, qright(self.mem.data.investments.InvestmentManager_with_investments_with_the_same_product(p).length()))
            opportunities=self.mem.con.cursor_one_field("select count(*) from opportunities where products_id=%s and executed is null and removed is null", (p.id, ))
            self.table.setItem(i, 6, qright(opportunities))
            self.table.setItem(i, 7, qright(p.splits.length()))
            self.table.setItem(i, 8, qright(p.estimations_dps.length()))
            self.table.setItem(i, 9, qright(p.estimations_eps.length()))

    def on_cmd_released(self):
        reply = QMessageBox.question(None, self.tr('Moving data between products'), self.tr("This action can't be undone.\nDo you want to continue?"), QMessageBox.Yes, QMessageBox.No)                  
        if reply==QMessageBox.Yes:
            self.mem.data.products.move_data_between_products(self.origin, self.destiny)
            self.origin.needStatus(3, downgrade_to=0)
            self.destiny.needStatus(3, downgrade_to=0)
            if self.chkInvestments.isChecked()==True:
                self.mem.data.investments.change_product_id(self.origin, self.destiny)
            self.mem.con.commit()
            self.reload()



from PyQt5.QtWidgets import QDialog,  QWidget
from decimal import Decimal
from xulpymoney.libxulpymoney import Dividend,  Money
from xulpymoney.libxulpymoneyfunctions import qmessagebox
from xulpymoney.libxulpymoneytypes import eProductType
from xulpymoney.ui.Ui_frmDividendsAdd import Ui_frmDividendsAdd

class frmDividendsAdd(QDialog, Ui_frmDividendsAdd):
    def __init__(self, mem, inversion, dividend=None,  parent=None):
        """
        Si dividend es None se insertar
        Si dividend es un objeto se modifica"""
        QWidget.__init__(self, parent)
        self.setupUi(self)
        self.mem=mem
        self.dividend=dividend
        self.investment=inversion
        
        self.neto=0
        self.tpc=0
        self.wdgDT.show_microseconds(False)
        self.wdgDT.show_timezone(False)
        self.lblGross.setText(self.tr("Gross in {}".format(self.investment.product.currency.symbol)))
        self.lblGrossAccount.setText(self.tr("Gross converted to {}".format(self.investment.account.currency.symbol)))
        if dividend==None:#insertar
            if self.investment.product.type.id in (eProductType.PrivateBond, eProductType.PublicBond):#Bonds
                self.mem.conceptos.load_bonds_qcombobox(self.cmb)
            elif self.investment.product.type.id in (eProductType.CFD,  eProductType.Future):
                self.mem.conceptos.load_futures_qcombobox(self.cmb)
            else:
                self.mem.conceptos.load_dividend_qcombobox(self.cmb)
            self.dividend=Dividend(self.mem)
            self.dividend.investment=inversion
            self.cmd.setText(self.tr("Add new dividend"))
            self.wdgDT.set(self.mem, None, self.mem.localzone)
            self.wdgCurrencyConversion.setConversion(Money(self.mem, self.txtBruto.decimal(), self.investment.product.currency), self.investment.account.currency, self.wdgDT.datetime(), None)
        else:#modificar 
            if self.investment.product.type.id in (eProductType.PrivateBond, eProductType.PublicBond):#Bonds
                self.mem.conceptos.load_bonds_qcombobox(self.cmb, self.dividend.concepto) 
            else:
                self.mem.conceptos.load_dividend_qcombobox(self.cmb, self.dividend.concepto) 
            self.wdgDT.set(self.mem, self.dividend.datetime, self.mem.localzone)
            self.wdgCurrencyConversion.setConversion(Money(self.mem, self.txtBruto.decimal(), self.investment.product.currency), self.investment.account.currency, self.wdgDT.datetime(), self.dividend.currency_conversion)
            self.txtBruto.setText(self.dividend.bruto)
            self.txtNeto.setText(self.dividend.neto)
            self.txtRetencion.setText(self.dividend.retencion)
            self.txtComision.setText(self.dividend.comision)
            self.txtDPA.setText(self.dividend.dpa)
            self.cmd.setText(self.tr("Edit dividend"))
 
    def on_txtBruto_textChanged(self):
        self.wdgCurrencyConversion.setConversion(Money(self.mem, self.txtBruto.decimal(), self.investment.product.currency), self.investment.account.currency, self.wdgDT.datetime(), self.wdgCurrencyConversion.factor)
        self.calcular()
        
    def on_txtRetencion_textChanged(self):
        self.calcular()
        
    def on_txtComision_textChanged(self):
        self.calcular()
        
    def calcular(self):
        try:
            if self.txtBruto.decimal()<Decimal(0):
                self.txtRetencion.setEnabled(False)
                self.txtDPA.setEnabled(False)
                self.txtComision.setEnabled(False)
                self.txtRetencion.setText(0)
                self.txtDPA.setText(0)
                self.txtComision.setText(0)
                self.neto=self.txtBruto.decimal()-self.txtComision.decimal()
                self.tpc=0
            else:
                self.txtRetencion.setEnabled(True)
                self.txtDPA.setEnabled(True)
                self.txtComision.setEnabled(True)
                self.neto=self.txtBruto.decimal()-self.txtRetencion.decimal()-self.txtComision.decimal()
                self.tpc=100*self.txtRetencion.decimal()/self.txtBruto.decimal()
            self.txtNeto.setText(self.neto)
            self.lblTPC.setText(self.tr("Withhonding tax retention percentage: {} %".format(self.tpc)))
            self.cmd.setEnabled(True)
        except:
            self.txtNeto.setText(self.tr("Calculation error"))
            self.lblTPC.setText(self.tr("Calculation error"))
            self.cmd.setEnabled(False)
 


    def on_cmd_pressed(self):
        concepto=self.mem.conceptos.find_by_id(self.cmb.itemData(self.cmb.currentIndex()))
        tipooperacion=concepto.tipooperacion
                        
        if tipooperacion.id==1 and (self.txtBruto.decimal()>Decimal('0') or self.txtNeto.decimal()>Decimal('0')):
            qmessagebox(self.tr("Expenses can't have a positive amount"))
            return
            
        if tipooperacion.id==2 and (self.txtBruto.decimal()<Decimal('0') or self.txtNeto.decimal()<Decimal('0')):
            qmessagebox(self.tr("Incomes can't have a negative amount"))
            return
        if self.txtRetencion.decimal()<Decimal('0') or self.txtDPA.decimal()<Decimal('0') or self.txtComision.decimal()<Decimal('0'):
            qmessagebox(self.tr("Retention, earnings por share and commission must be greater than zero"))
            return
        
        
        try:
            self.dividend.concepto=concepto
            self.dividend.bruto=self.txtBruto.decimal()
            self.dividend.retencion=self.txtRetencion.decimal()
            self.dividend.neto=self.neto
            self.dividend.dpa=self.txtDPA.decimal()
            self.dividend.datetime=self.wdgDT.datetime()
            self.dividend.comision=self.txtComision.decimal()
            self.dividend.currency_conversion=self.wdgCurrencyConversion.factor
        except:
            qmessagebox(self.tr("Data error. Please check them."))
            return

        
        self.dividend.save()
        self.mem.con.commit()
        self.done(0)



from PyQt5.QtWidgets import QDialog, QTableWidgetItem
from xulpymoney.ui.Ui_frmSelector import Ui_frmSelector
from xulpymoney.libxulpymoneyfunctions import qmessagebox

class frmSelector(QDialog, Ui_frmSelector):
    def __init__(self, mem, set,  selectedset, name = None, modal = False  ):
        """Lista es un objeto tipo set cuyos objetos agrupados tienen id y name
        Selected es otro objeto set"""
        QDialog.__init__(self)
        self.setupUi(self)   
        self.mem=mem
        self.selected=selectedset
        self.set=set
        
        tmpset=self.set.clone(self.mem)##Lo uso para no borrar mientras itero sale error
        for a in tmpset.arr:
            if self.selected.find_by_id(a.id)!=None:
                self.set.remove(a)
        
        self.load_tbl()
        self.load_tblSelected()

    def load_tblSelected(self):        
        self.tblSelected.setRowCount(self.selected.length())
        for i, l in enumerate(self.selected.arr):
            try:
                self.tblSelected.setItem(i, 0, QTableWidgetItem(str(l.id)))
                self.tblSelected.setItem(i, 1, QTableWidgetItem(l.name))
            except:
                qmessagebox(self.tr("This dialog got unknown selected data ({0}), select it again").format(str(self.selected)))
    
                self.selected=[]
                self.load_tbl()
                self.load_tblSelected()
                break
        
    def load_tbl(self):
        self.tbl.setRowCount(self.set.length())
        for i, l in enumerate(self.set.arr):
                self.tbl.setItem(i, 0, QTableWidgetItem(str(l.id)))
                self.tbl.setItem(i, 1, QTableWidgetItem(l.name))

    def on_cmdLeft_released(self):
        try:
            for i in self.tbl.selectedItems():
                self.set.selected=self.set.arr[i.row()]
            self.selected.append(self.set.selected)
            self.set.remove(self.set.selected)
        except:
            return
        self.load_tbl()
        self.load_tblSelected()
        
    def on_cmdRight_released(self):        
        try:
            for i in self.tblSelected.selectedItems():
                self.selected.selected=self.selected.arr[i.row()]
            self.set.append(self.selected.selected)       
            self.selected.remove(self.selected.selected) 
        except:
            return
        self.load_tbl()
        self.load_tblSelected()
        
        
    def on_cmd_released(self):
        self.done(0)
        
    def on_cmdUp_released(self):
        pos=None
        for i in self.tblSelected.selectedItems():
            pos=i.row()
        tmp=self.selected.arr[pos]
        self.selected.arr[pos]=self.selected.arr[pos-1]
        self.selected.arr[pos-1]=tmp
        self.load_tbl()
        self.load_tblSelected()             
        
    def on_cmdDown_released(self):
        pos=None
        for i in self.tblSelected.selectedItems():
            pos=i.row()
        tmp=self.selected.arr[pos+1]
        self.selected.arr[pos+1]=self.selected.arr[pos]
        self.selected.arr[pos]=tmp
        self.load_tbl()
        self.load_tblSelected()        
        
    def on_tbl_cellDoubleClicked(self, row, column):
        self.on_cmdLeft_released()
        
    def on_tblSelected_cellDoubleClicked(self, row, column):
        self.on_cmdRight_released()
        



import datetime
import logging
from PyQt5.QtCore import Qt, pyqtSlot,  QSize
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QDialog, QMenu,  QMessageBox, QVBoxLayout
from xulpymoney.libxulpymoney import Account, AccountOperation, Assets, Comment, InvestmentOperation, AccountOperationManager,  CreditCardOperationManager,  CreditCardOperation
from xulpymoney.libxulpymoneyfunctions import b2c,  c2b
from xulpymoney.libxulpymoneytypes import eComment
from xulpymoney.ui.Ui_frmAccountsReport import Ui_frmAccountsReport
from xulpymoney.ui.frmAccountOperationsAdd import frmAccountOperationsAdd
from xulpymoney.ui.frmCreditCardsAdd import frmCreditCardsAdd
from xulpymoney.ui.frmInvestmentOperationsAdd import frmInvestmentOperationsAdd
from xulpymoney.ui.wdgConceptsHistorical import wdgConceptsHistorical

class frmAccountsReport(QDialog, Ui_frmAccountsReport):
    def __init__(self, mem, account,  parent=None):
        """
            selIdAccount=None Inserción de cuentas
            selIdAccount=X. Modificación de cuentas cuando click en cmd y resto de trabajos"""
        QDialog.__init__(self, parent)
        self.setupUi(self)
        self.showMaximized()
        self.cmdDatos.setEnabled(False)     
        self.wdgDtPago.show_microseconds(False)
        self.wdgDtPago.show_timezone(False)
        self.wdgDtPago.setTitle(self.tr("Select payment time"))
        
        self.mem=mem
         
                
        self.account=account#Registro de account
        
        self.accountoperations=None#AccountOperationManager. Selected will be an AccountOperation
        self.creditcards=None#SetCreditCard. Selected will be a CreditCard
        self.creditcardoperations=CreditCardOperationManager(self.mem)#CreditCardOperationManager. Selected will be another CreditCardOperationManager
          
        self.tblOperaciones.settings(self.mem, "frmAccountsReport")
        self.tblCreditCards.settings(self.mem, "frmAccountsReport")
        self.tblCreditCardOpers.settings(self.mem, "frmAccountsReport")
        self.tblOpertarjetasHistoricas.settings(self.mem, "frmAccountsReport")
    
        self.wdgDtPago.set(self.mem, None, self.mem.localzone)
        
        self.mem.currencies.qcombobox(self.cmbCurrency)
        self.mem.data.banks_active().qcombobox(self.cmbEB)
                    
        if self.account==None:
            self.lblTitulo.setText(self.tr("New account data"))
            self.tab.setCurrentIndex(0)
            self.tab.setTabEnabled(1, False)
            self.tab.setTabEnabled(2, False)
            self.chkActiva.setChecked(Qt.Checked)
            self.chkActiva.setEnabled(False)
            self.cmdDatos.setText(self.tr("Add a new account"))
        else:               
            self.tab.setCurrentIndex(0)
            self.lblTitulo.setText(self.account.name)
            self.txtAccount.setText(self.account.name)
            self.txtNumero.setText(str(self.account.numero))            
            self.cmbEB.setCurrentIndex(self.cmbEB.findData(self.account.eb.id))
            self.cmbEB.setEnabled(False)    
            self.cmbCurrency.setCurrentIndex(self.cmbCurrency.findData(self.account.currency.id))
            self.cmbCurrency.setEnabled(False)
            self.chkActiva.setChecked(b2c(self.account.active))
            self.cmdDatos.setText(self.tr("Update account data"))

            anoinicio=Assets(self.mem).first_datetime_with_user_data().year       
            self.wdgYM.initiate(anoinicio,  datetime.date.today().year, datetime.date.today().year, datetime.date.today().month)
            self.on_AccountOperationChanged(None)
            self.on_CreditCardChanged(None)
            self.wdgYM.changed.connect(self.on_wdgYM_changed)

    @pyqtSlot() 
    def on_actionCreditCardAdd_triggered(self):
        w=frmCreditCardsAdd(self.mem,  self.account,  None, self)
        w.exec_()
        if w.result()==QDialog.Accepted:
            self.on_CreditCardChanged(w.creditcard)

    @pyqtSlot() 
    def on_actionCreditCardEdit_triggered(self):
        w=frmCreditCardsAdd(self.mem, self.account,  self.creditcards.selected, self)
        w.exec_()
        self.on_CreditCardChanged(self.creditcards.selected)

    @pyqtSlot() 
    def on_actionCreditCardActivate_triggered(self):
        if self.account.qmessagebox_inactive() or self.account.eb.qmessagebox_inactive():
            return

        self.creditcards.selected.active=not self.creditcards.selected.active
        self.creditcards.selected.save()
        self.mem.con.commit()
        self.on_CreditCardChanged(None)
                
    @pyqtSlot() 
    def on_actionCreditCardDelete_triggered(self):
        if self.creditcards.selected.is_deletable()==False:
            m=QMessageBox()
            m.setWindowIcon(QIcon(":/xulpymoney/coins.png"))
            m.setIcon(QMessageBox.Information)
            m.setText(self.tr("I can't delete the credit card, because it has dependent registers"))
            m.exec_()
        else:
            print("BORRANDO", self.creditcards.selected)
            self.mem.data.creditcards.delete(self.creditcards.selected)#self.creditcards is only a clone and is reloaded later
            self.mem.con.commit()
            self.on_CreditCardOperationChanged(None)

    def on_chkCreditCards_stateChanged(self, state):      
       self.on_CreditCardChanged(None) 

    def on_cmdDatos_released(self):
        id_entidadesbancarias=int(self.cmbEB.itemData(self.cmbEB.currentIndex()))
        cuenta=self.txtAccount.text()
        numerocuenta=self.txtNumero.text()
        active=c2b(self.chkActiva.checkState())
        currency=self.cmbCurrency.itemData(self.cmbCurrency.currentIndex())

        if self.account==None:
            cu=Account(self.mem, cuenta, self.mem.data.banks_active().find_by_id(id_entidadesbancarias), active, numerocuenta, self.mem.currencies.find_by_id(currency), None)
            cu.save()
            self.mem.data.accounts.append(cu) #Always to active
        else:
            self.account.eb=self.mem.data.banks_active().find_by_id(id_entidadesbancarias)
            self.account.name=cuenta
            self.account.numero=numerocuenta
            self.account.active=active
            self.account.currency=self.mem.currencies.find_by_id(currency)
            self.account.save()
            self.lblTitulo.setText(self.account.name)
        self.mem.con.commit()
        
        if self.account==None:
            self.done(0)
        self.cmdDatos.setEnabled(False)   

    @pyqtSlot()
    def on_wdgYM_changed(self):
        self.on_AccountOperationChanged(None)
        
    def on_AccountOperationChanged(self, o):
        """
            o=None, significa que hay que actualizarlo las opercuentas sin seleccionar
            o=AccountOperation, significa que hay que actualizar opercuentas y seleccionarla
        """
        lastMonthBalance=self.account.balance(datetime.date(self.wdgYM.year, self.wdgYM.month, 1)-datetime.timedelta(days=1), type=2)     
        self.accountoperations=AccountOperationManager(self.mem)           
        self.accountoperations.load_from_db(self.mem.con.mogrify("select * from opercuentas where id_cuentas=%s and date_part('year',datetime)=%s and date_part('month',datetime)=%s order by datetime, id_opercuentas", [self.account.id, self.wdgYM.year, self.wdgYM.month]))
        if o!=None:
            self.accountoperations.setSelected([o, ])
        self.accountoperations.myqtablewidget_lastmonthbalance(self.tblOperaciones,  self.account,  lastMonthBalance)   

    def on_CreditCardChanged(self, o):
        """
            o=None, significa que hay que actualizar las tarjetas sin seleccionar
            o=CreditCard, significa que hay que actualizar las tarjetas seleccionandola 
        """           
        if self.chkCreditCards.checkState()==Qt.Unchecked:
            self.creditcards=self.mem.data.creditcards_active().clone_of_account(self.account)
        else:
            self.creditcards=self.mem.data.creditcards_inactive().clone_of_account(self.account)  

        self.tabOpertarjetasDiferidas.setCurrentIndex(0)

        if o==None:#No selected
            self.creditcards.myqtablewidget(self.tblCreditCards)
            self.tblCreditCardOpers.setRowCount(0)
        else:
            self.creditcards.setSelected(o)
            self.tabOpertarjetasDiferidas.setEnabled(self.creditcards.selected.pagodiferido)
            self.creditcards.myqtablewidget(self.tblCreditCards)
            if self.creditcards.selected.pagodiferido==True:#Solo se muestran operaciones si es diferido
                self.creditcardoperations.load_from_db(self.mem.con.mogrify("select * from opertarjetas where id_tarjetas=%s and pagado=false", [self.creditcards.selected.id, ]))
                self.creditcardoperations.myqtablewidget(self.tblCreditCardOpers)
                self.grpPago.setEnabled(False)
            else:
                self.tblCreditCardOpers.setRowCount(0)

    def on_CreditCardOperationChanged(self, o):
        """ ES DISTINTA A CREDIRCARDCHANGED NO JUNTAR
            o=None, significa que hay que actualizar las tarjetas sin seleccionar
            o=CreditCardOperation, significa que hay que actualizar creditcardoperation, usando el id de la tarjeta seleccionada
        """          
        if self.chkCreditCards.checkState()==Qt.Unchecked:
            self.creditcards=self.mem.data.creditcards_active().clone_of_account(self.account)
        else:
            self.creditcards=self.mem.data.creditcards_inactive().clone_of_account(self.account)  
        self.creditcards.selected=o.tarjeta
        self.tabOpertarjetasDiferidas.setCurrentIndex(0)

        if o==None:#No selected
            self.creditcards.myqtablewidget(self.tblCreditCards)
            self.tblCreditCardOpers.setRowCount(0)
            self.grpPago.setEnabled(False)
        else:
            self.tabOpertarjetasDiferidas.setEnabled(self.creditcards.selected.pagodiferido)
            self.creditcards.myqtablewidget(self.tblCreditCards)
            if self.creditcards.selected.pagodiferido==True:#Solo se muestran operaciones si es diferido
                self.grpPago.setEnabled(True)
                self.creditcardoperations.load_from_db(self.mem.con.mogrify("select * from opertarjetas where id_tarjetas=%s and pagado=false", [self.creditcards.selected.id, ]))
                if o.__class__==CreditCardOperation:#It's an array
                    sel=CreditCardOperationManager(self.mem)
                    sel.append(o)
                    self.creditcardoperations.selected=sel
                self.creditcardoperations.myqtablewidget(self.tblCreditCardOpers)
            else:
                self.grpPago.setEnabled(False)
                self.tblCreditCardOpers.setRowCount(0)

    @pyqtSlot() 
    def on_actionOperationAdd_triggered(self):
        w=frmAccountOperationsAdd(self.mem, self.account)
        w.AccountOperationChanged.connect(self.on_AccountOperationChanged)
        w.exec_()

    @pyqtSlot() 
    def on_actionTransferDelete_triggered(self):
        """
            Ya está validado si es Comment coded 10001,10002,10003
        """
        args=Comment(self.mem).getArgs(self.accountoperations.selected.only().comentario)#origin,destiny,comission
        aoo=AccountOperation(self.mem, args[0])
        aod=AccountOperation(self.mem, args[1])

        message=self.tr("Do you really want to delete transfer from {0} to {1}, with amount {2} and it's commision?").format(aoo.account.name, aod.account.name, aoo.importe)
        reply = QMessageBox.question(self, 'Message', message, QMessageBox.Yes, QMessageBox.No)
        if reply == QMessageBox.Yes:
            if args[2]!=-1:
                aoc=AccountOperation(self.mem, args[2])
                aoc.borrar()
            aoo.borrar()
            aod.borrar()
            self.mem.con.commit()
            self.on_AccountOperationChanged(None)
        
    @pyqtSlot() 
    def on_actionOperationEdit_triggered(self):
        w=frmAccountOperationsAdd(self.mem, self.account, self.accountoperations.selected.only())
        w.AccountOperationChanged.connect(self.on_AccountOperationChanged)
        w.exec_()

    @pyqtSlot() 
    def on_actionOperationDelete_triggered(self):
        self.accountoperations.selected.only().borrar() 
        self.mem.con.commit()         
        self.on_AccountOperationChanged(None)

    @pyqtSlot() 
    def on_actionCreditCardOperAdd_triggered(self):
        w=frmAccountOperationsAdd(self.mem, self.account, None, self.creditcards.selected)
        w.CreditCardOperationChanged.connect(self.on_CreditCardOperationChanged)
        w.AccountOperationChanged.connect(self.on_AccountOperationChanged)#Tarjetas en debito
        w.exec_()
            
    @pyqtSlot() 
    def on_actionCreditCardOperEdit_triggered(self):
        #Como es unico
        selOperCreditCard=self.creditcardoperations.selected.arr[0]
        w=frmAccountOperationsAdd(self.mem,  self.account, None, self.creditcards.selected, selOperCreditCard)
        w.CreditCardOperationChanged.connect(self.on_CreditCardOperationChanged)
        w.exec_()

    @pyqtSlot() 
    def on_actionCreditCardOperDelete_triggered(self):
        for o in self.creditcardoperations.selected.arr:
            o.borrar()
            self.creditcardoperations.arr.remove(o)
        self.mem.con.commit()
        self.on_CreditCardChanged(self.creditcards.selected)
        
    @pyqtSlot() 
    def on_actionCreditCardOperRefund_triggered(self):
        w=frmAccountOperationsAdd(self.mem, opertarjeta=self.creditcardoperations.selected.arr[0], refund=True)
        w.CreditCardOperationChanged.connect(self.on_CreditCardOperationChanged)
        w.exec_()
            
    @pyqtSlot() 
    def on_actionInvestmentOperationDelete_triggered(self):
        investmentoperation=InvestmentOperation(self.mem).init__from_accountoperation(self.accountoperations.selected)
        investmentoperation.investment.op.remove(investmentoperation)
        self.mem.con.commit()     
        print("Borrando investment ooperation",  investmentoperation)
        self.on_AccountOperationChanged(None)


    @pyqtSlot() 
    def on_actionInvestmentOperationEdit_triggered(self):
        investmentoperation=InvestmentOperation(self.mem).init__from_accountoperation(self.accountoperations.selected.only())
        w=frmInvestmentOperationsAdd(self.mem, investmentoperation.investment, investmentoperation, self)
        w.exec_()
        logging.debug("Edit investmentoperation {}".format(self.accountoperations.selected.only()))
        self.on_AccountOperationChanged(self.accountoperations.selected.only())

    @pyqtSlot()
    def on_actionConceptReport_triggered(self):
        if self.tab.currentIndex()==0:
            concepto=self.accountoperations.selected.only().concepto
        else:
            concepto=self.creditcardoperations.selected.first().concepto

        d=QDialog(self)     
        d.resize(self.mem.settings.value("frmAccountsReport/qdialog_conceptreport", QSize(800, 600)))
        d.setWindowTitle(self.tr("Historical report of {}").format(concepto.name))
        w = wdgConceptsHistorical(self.mem, concepto, d)
        lay = QVBoxLayout(d)
        lay.addWidget(w)
        d.exec_()
        self.mem.settings.setValue("frmAccountsReport/qdialog_conceptreport", d.size())


    ## Selection can be only one row, due to table definitions
    def on_tblOperaciones_customContextMenuRequested(self,  pos):      
        if self.account.qmessagebox_inactive() or self.account.eb.qmessagebox_inactive():
            return

        if self.accountoperations.selected.length()==0:
            self.actionOperationDelete.setEnabled(False)
            self.actionOperationEdit.setEnabled(False)   
            self.actionTransferDelete.setEnabled(False)
            self.actionConceptReport.setEnabled(False)
        else:
            if self.accountoperations.selected.only().es_editable()==False:
                self.actionOperationDelete.setEnabled(False)
                self.actionOperationEdit.setEnabled(False)   
                #Una transferencia bien formada no es editable solo con transfer delete.
                if Comment(self.mem).getCode(self.accountoperations.selected.only().comentario) in (eComment.AccountTransferOrigin, eComment.AccountTransferDestiny, eComment.AccountTransferOriginCommission):
                    self.actionTransferDelete.setEnabled(True)
                else:
                    self.actionTransferDelete.setEnabled(False)
            else: #es editable
                self.actionOperationDelete.setEnabled(True)    
                self.actionOperationEdit.setEnabled(True)      
                self.actionTransferDelete.setEnabled(False)
            self.actionConceptReport.setEnabled(True)
            
        menu=QMenu()
        menu.addAction(self.actionOperationAdd)
        menu.addAction(self.actionOperationEdit)
        menu.addAction(self.actionOperationDelete)
        menu.addSeparator()
        menu.addAction(self.actionConceptReport)
        menu.addSeparator()
        menu.addAction(self.actionTransferDelete)

        if self.accountoperations.selected.length()!=0:
            if self.accountoperations.selected.only().concepto.id in [29, 35]:#Shares sale or purchase, allow edit or delete investment operation
                menu.addSeparator()
                menu.addAction(self.actionInvestmentOperationEdit)
                menu.addAction(self.actionInvestmentOperationDelete)
        menu.exec_(self.tblOperaciones.mapToGlobal(pos))

    def on_tblOperaciones_itemSelectionChanged(self):
        self.accountoperations.selected.clean()
        for i in self.tblOperaciones.selectedItems():#itera por cada item no row.
            if i.column()==1 and i.row()!=0:#Initial month
                id=int(self.tblOperaciones.item(i.row(), 5).text())#Id it's hidden in the fifth column
                self.accountoperations.selected.append(self.accountoperations.find_by_id(id)) #AccountOperationManager is a DictManager
        print ("Seleccionado: " +  str(self.accountoperations.selected.dic))
        

    def on_tblCreditCards_customContextMenuRequested(self,  pos):
        if self.account.qmessagebox_inactive():
            return 
        menu=QMenu()
        menu.addAction(self.actionCreditCardOperAdd)
        menu.addSeparator()
        menu.addAction(self.actionCreditCardAdd)
        menu.addAction(self.actionCreditCardEdit)
        menu.addAction(self.actionCreditCardDelete)
        menu.addSeparator()
        menu.addAction(self.actionCreditCardActivate)
        
        if self.creditcards.selected==None:
            self.actionCreditCardDelete.setEnabled(False)
            self.actionCreditCardEdit.setEnabled(False)
            self.actionCreditCardActivate.setEnabled(False)
        else:
            self.actionCreditCardDelete.setEnabled(True)
            self.actionCreditCardEdit.setEnabled(True)
            self.actionCreditCardActivate.setEnabled(True)
            if self.creditcards.selected.active==True:
                self.actionCreditCardActivate.setChecked(True)
            else:
                self.actionCreditCardActivate.setChecked(False)
        menu.exec_(self.tblCreditCards.mapToGlobal(pos))



    def on_tblCreditCards_itemSelectionChanged(self):
        self.creditcards.selected=None
        for i in self.tblCreditCards.selectedItems():#itera por cada item no row.
            if i.column()==0:
                self.creditcards.setSelected(self.creditcards.arr[i.row()])
        
        self.on_CreditCardChanged(self.creditcards.selected)

    def on_tblCreditCardOpers_customContextMenuRequested(self,  pos):
        if self.account.qmessagebox_inactive() or self.account.eb.qmessagebox_inactive() or self.creditcards.selected.qmessagebox_inactive():
            return
        if self.creditcardoperations.selected!=None:
            if self.creditcardoperations.selected.length()!=1: # 0 o más de 1
                self.actionCreditCardOperDelete.setEnabled(False)
                self.actionCreditCardOperEdit.setEnabled(False)
                self.actionCreditCardOperRefund.setEnabled(False)
                self.actionConceptReport.setEnabled(False)
            else:
                self.actionCreditCardOperDelete.setEnabled(True)
                self.actionCreditCardOperEdit.setEnabled(True)
                if self.creditcards.selected.pagodiferido==True and self.creditcardoperations.selected.arr[0].importe<0:#Only difered purchases
                    self.actionCreditCardOperRefund.setEnabled(True)
                else:
                    self.actionCreditCardOperRefund.setEnabled(False)
                self.actionConceptReport.setEnabled(True)
            
        menu=QMenu()
        menu.addAction(self.actionCreditCardOperAdd)
        menu.addAction(self.actionCreditCardOperEdit)
        menu.addAction(self.actionCreditCardOperDelete)
        menu.addSeparator()
        menu.addAction(self.actionConceptReport)
        menu.addSeparator()
        menu.addAction(self.actionCreditCardOperRefund)
        menu.exec_(self.tblCreditCardOpers.mapToGlobal(pos))


    def on_tblCreditCardOpers_itemSelectionChanged(self):
        self.creditcardoperations.selected=CreditCardOperationManager(self.mem)
        for i in self.tblCreditCardOpers.selectedItems():#itera por cada item no row.
            if i.column()==0:
                self.creditcardoperations.selected.append(self.creditcardoperations.arr[i.row()])  
        
        #Activa el grp Pago
        if self.creditcardoperations.selected.length()>0:
            self.grpPago.setEnabled(True)
        else:
            self.grpPago.setEnabled(False)

        #Calcula el balance
        self.lblPago.setText(self.mem.localcurrency.string(self.creditcardoperations.selected.balance()))
 
    def on_cmdPago_released(self):
        c=AccountOperation(self.mem, self.wdgDtPago.datetime(), self.mem.conceptos.find_by_id(40), self.mem.tiposoperaciones.find_by_id(7), self.creditcardoperations.selected.balance(), "Transaction in progress", self.account, None)
        c.save()
        
        c.comentario=Comment(self.mem).encode(eComment.CreditCardBilling, self.creditcards.selected, c)
        c.save()
        
        #Modifica el registro y lo pone como pagado y la datetime de pago y añade la opercuenta
        for o in self.creditcardoperations.selected.arr:
            o.fechapago=self.wdgDtPago.datetime()
            o.pagado=True
            o.opercuenta=c
            o.save()
        self.mem.con.commit()
        self.on_AccountOperationChanged(None)
        self.on_CreditCardChanged(self.creditcards.selected)

    
    def on_cmdDevolverPago_released(self):
        id_opercuentas=self.cmbFechasPago.itemData(int(self.cmbFechasPago.currentIndex()))
        cur = self.mem.con.cursor()      
        cur.execute("delete from opercuentas where id_opercuentas=%s", (id_opercuentas, ))#No merece crear objeto
        cur.execute("update opertarjetas set fechapago=null, pagado=false, id_opercuentas=null where id_opercuentas=%s", (id_opercuentas, ) )
        self.mem.con.commit()
        cur.close()     
        self.on_AccountOperationChanged(None)
        self.on_CreditCardChanged(self.creditcards.selected)
        self.tabOpertarjetasDiferidas.setCurrentIndex(0)     
        
    @pyqtSlot(int) 
    def on_cmbFechasPago_currentIndexChanged(self, index):
        if index==-1:#Empty
            self.tblOpertarjetasHistoricas.setRowCount(0)
            return
        id_opercuentas=self.cmbFechasPago.itemData(index)
        setPaidCreditCardOperations=CreditCardOperationManager(self.mem)
        setPaidCreditCardOperations.load_from_db("select * from opertarjetas where id_opercuentas={};".format(id_opercuentas, ))
        setPaidCreditCardOperations.myqtablewidget(self.tblOpertarjetasHistoricas)

    def on_tabOpertarjetasDiferidas_currentChanged(self, index): 
        if  index==1: #PAGOS
            #Carga combo
            self.cmbFechasPago.clear()
            cur = self.mem.con.cursor()       
            cur.execute("select distinct(fechapago), id_opercuentas from opertarjetas where id_tarjetas=%s and fechapago is not null  order by fechapago;", (self.creditcards.selected.id, ))
            for row in cur:   
                ao=AccountOperation(self.mem, row['id_opercuentas'])
                self.cmbFechasPago.addItem(self.tr("{0} was made a paid of {1}").format(str(row['fechapago'])[0:19],  self.mem.localcurrency.string(-ao.importe))    , ao.id)
            self.cmbFechasPago.setCurrentIndex(cur.rowcount-1)
            cur.close()     

    def on_txtAccount_textChanged(self):
        self.cmdDatos.setEnabled(True)
    def on_txtNumero_textChanged(self):
        self.cmdDatos.setEnabled(True)
    def on_cmbEB_currentIndexChanged(self, index):
        self.cmdDatos.setEnabled(True)
    def on_chkActiva_stateChanged(self,  state):
        self.cmdDatos.setEnabled(True)
